<?php
require_once('inc/folder.php');
require_once('inc/cache/cache.php');
require_once('inc/rss/item.php');


/**
  @class CacheFolder
  @brief Class representing a cached folder.
  @author Martin Zlomek
  @date 02-03-2006
 */
abstract class CacheFolder extends Folder {
	/** Parent account object. */
	public $account;

	/** Internal ID of the folder. */
	public $folderID;

	/**
	  @brief Create a new folder object.
	  @param $account Parent account object of the folder.
	  @param $folderID ID of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */
	protected function __construct(&$account, $folderID, $name, $rights, $sync)
	{

    $this->sync = $sync?true:false;
		$this->account	= &$account;
		$this->folderID	= $folderID;
		
    parent::__construct($name, $rights);
	}

	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	  @return ID of the folder.
	 */
	protected static function create($account, $name, $rights)
	{
		$parentFolderName = preg_replace('/\/[^\/]*$/', '', $name);
		
		if (isset($account->folders['main'][$parentFolderName]))
			$parentFolderID = $account->folders['main'][$parentFolderName]->folderID;
		else
			$parentFolderID = null;

		$cache = Cache::instance($account->user);
		return $cache->createFolder(array('parent_folder_id' => $parentFolderID,
										                  'account_id'		=> $account->accountID,
										                  'name'			 => $name,
										                  'rights'			=> $rights));
	}

	/**
	  @brief Delete the folder.
	 */
	public function delete($subfolders = 0)
	{
		$account = &$this->account;

		$cache = Cache::instance($account->user);
		$cache->deleteFolder($account->accountID, $this->folderID, $subfolders);
	}
	
	public function deleteItems($oItems = false)
	{
	  $account = &$this->account;
		
    $cache = Cache::instance($account->user);
		$cache->deleteItems($this,$oItems);

	}
	
	public function deleteOlder($sDays)
	{
	  $account = &$this->account;
		
    $cache = Cache::instance($account->user);
    $sTime = time()-$sDays*86400;  
    
    $aFilter['tag'] = 'id,rid,date';
    $aFilter['sql'] = ' date < '.$sTime;
    
		$aOldItems = $this->getItems($aFilter);

    if($aOldItems) 
      $this->deleteItems($aOldItems);
	}
	
  public function moveItems($oFolder,$oItems = false,$aMovedID = false)
  {
      $cache = Cache::instance($this->account->user);
      $cache->moveItems($this,$oFolder,$oItems,$aMovedID);	  
  }
  
  public function copyItems($oFolder,$oItems = false,$aCopiedID = false)
  {
      $cache = Cache::instance($this->account->user);
      $cache->copyItems($this,$oFolder,$oItems,$aCopiedID);	  
  }
	/**
	  @brief Empty the folder.
	 */
	protected function deleteAllItems()
	{
		$account = &$this->account;

		$cache = Cache::instance($account->user);
		$cache->emptyFolder($account->accountID,$this->folderID);
	}
    
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	protected function rename($newName)
	{
		$account = $this->account;

		$cache = Cache::instance($account->user);
		$cache->renameFolder($account->accountID, $this->folderID,$this->name,$newName);

		$this->name = $newName;
	}
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	public function setSubscription($state)
	{
		$account = $this->account;

		$cache = Cache::instance($account->user);
		
    $cache->updateFolder($this->folderID,array('sync'=>$state));

		$this->sync = $state?true:false;
	}	


	/**
	  @brief Get the list of folder items.
	  @param $class Class of the item objects.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of $class objects.
	 */
	public function getItems($class, $filter, $orderBy, $offset, $limit,$fields ='',$search = false)
	{
	  $account = &$this->account;

		$cache = Cache::instance($account->user);
		return $cache->getItems($this, $class, $filter, $orderBy, $offset, $limit,$fields,$search);
	}

	/**
	  @brief Get an item.
	  @param $class Class of the item object.
	  @param $itemID ID of the item.
	  @return $class object.
	 */
	public function getItem($class, $itemID,$cache = array())
	{
		$account = &$this->account;

		if (!isset($cache['cache'])){
			$cache = array();
			$cache['cache'] = Cache::instance($account->user);
			$fCache = false;
		}else $fCache = true;
		return $cache['cache']->getItem($this, $class, $itemID, $fCache);
	}




	/**
	  @brief Load direct childern of the folder
	  @param $class Class of the folder objects.
	 */	
	public function getChildern($account)
	{
	     $account = &$this->account;

       $cache = Cache::instance($account->user);
	     return $cache->getChildern($account, 
                                  $account->folderClassName, 
                                  $this->folderID);
  }
  
	/**
	  @brief Load direct parent of the folder
	  @param $class Class of the folder objects.
	 */	
	public function getParent($account = false)
	{
	     $account = &$this->account;
		
       $cache = Cache::instance($account->user);
	     return $cache->getParent($account,
                                $account->classPrefix.'Folder',
                                $this->folderID);
  }
	
	/**
	  @brief Load direct parent of the folder
	  @param $class Class of the folder objects.
	 */	
	protected function getGWFolder($name)
	{
		try {
		  if ($this->account->gwAccount)
			 $folder = $this->account->gwAccount->getFolder($name); 
			else
			 $folder = false;
		} catch (Exc $e) {
			$folder = false;
		}
		return $folder;
	}


	/**
	  @brief Get number of items.
	  @param $flags Flags of counted items.
	  @param $positive If false, negative value of Flags will be used
	  @return The number of items.
	 */
	public function countItems($flags = 0, $positive = true,$filter = "",$search = false)
	{
		$account = &$this->account;
    $cache = Cache::instance($account->user);
		return $cache->countItems($this, $flags, $positive,$filter,$search);
	}
	
	public function markItems($flag)
	{
    $account = &$this->account;
    $cache = Cache::instance($account->user);
    $cache->markFolderItems($this->folderID,Item::FLAG_SEEN);
  }
  
  public function unmarkItems($flag)
  {
    $account = &$this->account;
    $cache = Cache::instance($account->user);
    $cache->unmarkFolderItems($this->folderID,Item::FLAG_SEEN);
  }
  
/**
	  @brief Synchronize the item cache with the list of items.
	  @param $class Class of the item object.
	  @param $aItemsID The list of item id's.
	 */
	protected function sync($class, $aItemsID,$folder)
	{
		$account = $this->account;

		//Retrieve cached items
		$cache['cache'] = Cache::instance($account->user);	

		$aCacheItemsID = $cache['cache']->getItemsID($this->folderID);
    $aToSyncItems = array();
    
		//Get instance of imap to retrieve item details
		if ($class=='IMAPItem') {
			$protocolItem = IMAP::instance($account);
		} else if ($class=='LocalPOPItem') {
			$protocolItem = LocalPOP::instance($account);
		} else if ($class =="POP3Item"){
      $protocolItem = POP3::instance($account);
    } else if ($class=="RSSItem"){
      $protocolItem = RSS::instance($account);
    }
		if ($aItemsID){
     $cache['cache']->transaction(); 
     foreach ($aItemsID as $oItem) {      
      if($class=='IMAPItem'){
      	$oItem->uid = IMAP::fixID($oItem->uid);
      }
    //Item exists in databas - check for flags if imap item  
			if (isset($aCacheItemsID[$oItem->uid])) {
				$itemID = $aCacheItemsID[$oItem->uid];
				if ($class=='IMAPItem'){
				  $flags = IMAPItem::decodeFlags($oItem);
				  
				  if($flags & ITEM::FLAG_FLAGGED){
				    if($itemID['color']=='Z')
				      $itemID['color'] = 1;
          }else{
            $itemID['color'] = 'Z';
          }

          if (isset($flags)) {
					 // keep flags higher than 64 (they are not stored to IMAP, just kept in database)
					 $newFlags = $flags | ($itemID['flags'] & ~63);
					 if ($newFlags != $itemID['flags'])
						  $cache['cache']->updateItem($itemID['item_id'], array('flags' => $newFlags,'color'=>$itemID['color']),$cache);	
				    }
				}
				//Remove item from cacheditems ( remaining items in array will be
				//deleted if they are no longer in mailbox)
				unset($aCacheItemsID[$oItem->uid]);
        
			} else {
				//Append item to synchronization queue
				$aToSyncItems[] = $oItem;  
			}
		}
    $cache['cache']->commit(); 
  }

  	//Do chunked synchronization
		if ($aToSyncItems){ 
		  if ($class!="RSSItem")
        $lower = $protocolItem->msgNumber($aToSyncItems[0]->uid,$aItemsID);//!
		    	
      $aChunkedItems = array_chunk($aToSyncItems,CHUNK_SIZE);

      if ($aChunkedItems) foreach($aChunkedItems as $aSyncItems){
			$aMailItems = array();
				$cache['cache']->transaction();
				
        $upper = (($lower+CHUNK_SIZE-1)>($lower+count($aToSyncItems)-1))?($lower+count($aToSyncItems)-1):($lower+CHUNK_SIZE-1);
        
    		if ($class=='LocalPOPItem'){
          foreach($aSyncItems as $k=>$v)
						$aMailItems[$k] = $protocolItem->getItem($this->name,$v->uid);
				}else if ($class=='POP3Item'){
          foreach($aSyncItems as $k=>$v)
						$aMailItems[$k] = $protocolItem->getItemHeaders($this->name,$v->uid);       
        } else if ($class=='IMAPItem'){
					$firstID = $aSyncItems[0]->uid;
					$lastID = $aSyncItems[count($aSyncItems)-1]->uid;
					$aMailItems = $protocolItem->getItems($this->name,$firstID.":".$lastID,FT_UID);
					
					
        	/*$aMailItems = $protocolItem->getItems($this->name,$lower.":".$upper);*/
        }else if ($class=="RSSItem"){
          $aMailItems = $aSyncItems;
        }
        
				foreach($aMailItems as $oSyncItem){
          if($oSyncItem){  
            call_user_func(array($class, 'decode'), $oSyncItem,false,$this,$protocolItem);  
  					if($class=='IMAPItem'){
            			$sRID = IMAP::fixID($oSyncItem->uid);
            		}else{
            			$sRID = $oSyncItem->uid;
            		}
  					//Check subject encoding and try to convert it to UTF-8
  					$cache['cache']->createItem(array('folder_id' => $this->folderID,
    										 'rid'     => $sRID,
    										 'size'    => $oSyncItem->size,
    										 'date'    => $oSyncItem->date,
    										 'flags'   => $oSyncItem->flags?$oSyncItem->flags:0,
    										 'header_from' => $oSyncItem->from,
    										 'header_to'   => $oSyncItem->to,
    										 'subject' =>$oSyncItem->subject,
    										 'static_flags'	 => $oSyncItem->staticFlags,
    										 'priority'	 => $oSyncItem->priority,
    										 'color'	 => $oSyncItem->color,
    										 'smime_status' => $oSyncItem->sMimeStatus,
    										 'has_attachment' => $oSyncItem->hasAttachments ? 'T' : 'F',
    										 'body'=>$oSyncItem->body ? $oSyncItem->body : NULL),
                         $cache);
  					
					}
					unset($aCacheItemsID[$key]);
				}
				$cache['cache']->commit();
				$lower = ($upper+1);
			}
		}

		//remove from sync list in lpop
		$protocolItem->drop($this->name);

		// Delete items from the item cache. 
		if ($aCacheItemsID){          
			$cache['cache']->transaction();
			foreach ($aCacheItemsID as $itemID)  {
        if($class=='RSSItem' && !($itemID['flags'] & Item::FLAG_DELETED)){
            continue;
        }
        $cache['cache']->deleteItem($account->accountID, $folder->folderID,$itemID['item_id'],$cache);
      }
			$cache['cache']->commit();
		}
  }
  
  //Classic search that returns 
  public function search($sPhrase,$aFilterTag){
    $cache = Cache::instance($this->account->user);	
    $oItems = $this->getItems($aFilterTag);

    return $oItems;
  }
    
	public function updateSearchFilter($result,$aFilterTag,$syncField = 'RID'){
    if($result){
      if(count($result)>DB_MAX_IDS_IN_QUERY)
        $resultCount = DB_MAX_IDS_IN_QUERY;
      else 
        $resultCount = count($result);
    
   		$sql = '';
   	if($resultCount){
		$sql = "(";
		for($i=0;$i<$resultCount;$i++){
			$sql .= (($i>0)?"OR":"")." ".$syncField." = '".$result[$i]."' ";
		}
		$sql .= ")";
      }
      if($aFilterTag['sql']){
      	$aFilterTag['sql'] = " ( ".$aFilterTag['sql']." OR ".$sql." ) ";
      }else{
      	$aFilterTag['sql'] = $sql;
      }
    }
    unset($aFilterTag['fulltext']);
    return $aFilterTag;
  }
}

require_once('inc/imap/folder.php');
require_once('inc/localpop/folder.php');
require_once('inc/pop3/folder.php');

?>