<?php

require_once('inc/item.php');
require_once('inc/cache/cache.php');

/**
  @class CacheItem
  @brief Class representing a cached item.
  @author Martin Zlomek
  @date 02-03-2006
 */
class CacheItem extends Item {
	/** Parent folder object. */
	public $folder;

	/** Internal ID of the item. */
	public $itemID;

	/** Remote ID of the item. */
	public $rid;

	/**
	  @brief Create a new item object.
	  @param $folder Parent folder object of the item.
	  @param $itemID ID of the item.
	  @param $rid Remote ID of the item.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	protected function __construct($folder, $itemID, $rid, $size, $date, 
						$flags, $from, $to, $subject, $staticFlags, $priority, $color,
						$sMimeStatus, $hasAttachments)
	{
		parent::__construct($size, $date, $flags, $from, $to, $subject, 
						$staticFlags, $priority, $color, $sMimeStatus, $hasAttachments);

		$this->folder = $folder;
		$this->itemID = $itemID;
		$this->rid	= $rid;
		$this->size = $size;

	}

	/**
	  @brief Create a new item.
	  @param $folder Parent folder object of the item.
	  @param $item The item.
	  @param $message The message of the item.
	  @return ID of the item.
	 */

	//extends from protected static function create($folder, $message) todo: synchronise, strict 
	protected static function create($folder, $item, $message,$cache = array(),$file = false)
	{
		$account = $folder->account;

    if (!$cache['cache'])
		$cache['cache'] = Cache::instance($account->user);
		if($folder->itemClassName=='IMAPItem'){
			$item->uid = IMAP::fixID($item->uid);
		}
		$itemID = $cache['cache']->createItem(array('folder_id'	=> $folder->folderID,
											'rid'		 => $item->uid,
											'size'		=> $item->size,
											'date'		=> $item->date,
											'flags'		=> $item->flags,
											'header_from' => $item->from,
											'header_to'	=> $item->to,
											'subject'	 => $item->subject,
											'static_flags' => $item->staticFlags,
											'priority'	 => $item->priority,
											'color'	 => $item->color,
											'smime_status' => $item->sMimeStatus,
											'has_attachment' => $item->hasAttachments ? 'T' : 'F'),
                      $cache);

		$cache['cache']->createMessage($account->accountID,
							  $folder->folderID,
							  $itemID,
							  $message,
                $file);

		return $itemID;
	}
	
	/**
	  @brief Edit an item.
	  @param $item The item.
	  @param $message The message of the item.
	  @param $cache Cache object related to logged user.
	 */
	protected function edit($item, $message, $cache = array(),$file = false)
	{
		$folder  = $this->folder;
		$account = $folder->account;

		if (!$cache['cache'])
			$cache['cache'] = Cache::instance($account->user);

		$this->editProperties($item, $cache);
		
		$cache['cache']->editMessage($account->accountID,
							  $folder->folderID,
							  $this->itemID,
							  $message,
                $file);
	}
	
	/**
	  @brief Edit item properties.
	  @param $item StdObject containing properties that should be changed.
	  @param $cache Cache object related to logged user.
	 */
	private function editProperties($item, $cache = array()) {
		if (!$cache['cache']) {
			$account = $this->folder->account;
			$cache['cache'] = Cache::instance($account->user);
		}

		// prepare array of all properties that can be edited
		$allowedItems = array('rid'		 => 'uid',
										'size'		=> 'size',
										'date'		=> 'date',
										'flags'		=> 'flags',
										'header_from' => 'from',
										'header_to'	=> 'to',
										'subject'	 => 'subject',
										'static_flags' => 'staticFlags',
										'priority'	 => 'priority',
										'color'	 => 'color',
										'smime_status' => 'sMimeStatus',
										'has_attachment' => 'hasAttachments');
		
		$setItems = array();
		foreach ($allowedItems as $key => $value) {
			if (isset($item->$value) && $item->$value!='') {
				 $setItems[$key] = $item->$value;
				 $this->$value = $item->$value;
			}
		}
		
		// handle special case
		if (isset($setItems['has_attachment']))
			$setItems['has_attachment'] = $setItems['has_attachment'] ? 'T' : 'F';

		$cache['cache']->updateItem($this->itemID, $setItems, $cache);
	}
	
	public function updateItem($fields){
    $cache['cache'] = Cache::instance($this->folder->account->user);
    $cache['cache']->updateItem($this->itemID, $fields, $cache);
  }

	/**
	  @brief Delete the item.
	 */
	protected function delete($cache = array())
	{
		$folder  = $this->folder;
		$account = $folder->account;

    if (!$cache['cache']){
      $fCache = false;
		  $cache['cache'] = Cache::instance($account->user);
		} else $fCache = true;
		$cache['cache']->deleteItem($account->accountID, $folder->folderID, $this->itemID,$fCache);
	}

	/**
	  @brief Move an item to another folder.
	  @param $folder Destination folder object.
	 */
	protected function move($folder,$cache = array())
	{

    $account = $folder->account;

    if (!$cache['cache'])
		  $cache['cache'] = Cache::instance($account->user);
		  
		$cache['cache']->updateItem($this->itemID, array('folder_id' => $folder->folderID,
												'rid'		=> $this->rid), $cache);
												
		if (strtolower($account->protocol)=='imap')
      $cache['cache']->moveMessage($account->accountID,
                          $this->folder->folderID,
							            $this->itemID,
                          $folder->folderID);

		$this->folder = &$folder;
	}
	
	/**
	  @brief Copy an item to another folder.
	  @param $folder Destination folder object.
	  @param $message Original message
	 */
	protected function copy($folder, $message = '', $cache = array())
	{
		$account = $folder->account;
		if(!$cache['cache'])
       $cache['cache'] = Cache::instance($account->user);
 
		$itemID = $cache['cache']->createItem(array('folder_id'	=> $folder->folderID,
											'rid'		 => $this->rid,
											'size'		=> $this->size,
											'date'		=> $this->date,
											'flags'		=> $this->flags,
											'header_from' => $this->from,
											'header_to'	=> $this->to,
											'subject'	 => $this->subject,
											'static_flags' => $this->staticFlags,
											'priority'	 => $this->priority,
											'color'	 => $this->color,
											'smime_status' => $this->sMimeStatus,
											'has_attachment' => $this->hasAttachments ? 'T' : 'F'),
                      $cache);

		if (strtolower($account->protocol)=='imap')
      $cache['cache']->createMessage($account->accountID,
            							  $folder->folderID,
            							  $itemID,
            							  $message);
	}

	/**
	  @brief Set flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function setFlags($flags, $field)
	{

		$folder  = $this->folder;
		$account = $folder->account;
		
		switch ($field) {
		case 'flags':
			$params = array('flags' => $this->flags | $flags);
			break;
		
		case 'staticFlags':
			$params = array('static_flags' => $this->staticFlags | $flags);
			break;
		
		default:
			throw new Exc('flags_unknown_field');//no rerror id
		}
		$cache = Cache::instance($account->user);
		$cache->updateItem($this->itemID, $params);

		parent::setFlags($flags, $field);
	}

	/**
	  @brief Unset flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function unsetFlags($flags, $field)
	{
		$folder  = $this->folder;
		$account = $folder->account;
		
		switch (strtolower($field)) {
		case 'flags':
			$params = array('flags' => $this->flags & ~$flags);
			break;
		
		case 'staticflags':
			$params = array('static_flags' => $this->staticFlags & ~$flags);
			break;
		
		default:
			throw new Exc('flags_unknown_field');
		}

		$cache = Cache::instance($account->user);
		$cache->updateItem($this->itemID, $params);

		parent::unsetFlags($flags, $field);
	}
	
	/**
	  @brief Set flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function changeColor($color)
	{
		$folder  = $this->folder;
		$account = $folder->account;
		
		// convert empty string or 0 to null
		if (!$color)
			$color = null;
		
		$cache = Cache::instance($account->user);
		$cache->updateItem($this->itemID, array('color' => $color));
		
		$this->color = $color;
		
		// set/unset also IMAP flagged state for the item
		if ($color !== null)
			$this->setFlags(Item::FLAG_FLAGGED);
		else
			$this->unsetFlags(Item::FLAG_FLAGGED);
	}


	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	protected function getMessage()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		$cache = Cache::instance($account->user);
		return $cache->getMessage($account->accountID,
								  $folder->folderID,
								  $this->itemID);
	}
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function getMessageFile()
	{
		$folder  = $this->folder;
		$account = $folder->account;

		$cache = Cache::instance($account->user);
		return $cache->getMessageFile($cache->getMessageDir($account->accountID, $folder->folderID),
								  $this->itemID);
	}

	/**
	  @brief Cache the message of the item.
	  @param $message The message.
	 */
	protected function createMessage($message)
	{
		$folder  = $this->folder;
		$account = $folder->account;

		$cache = Cache::instance($account->user);
		$cache->createMessage($account->accountID,
							  $folder->folderID,
							  $this->itemID,
							  $message);
	}
	
	/**
	  @brief Get the structure of message of the item.
	  @param $decodeMime Bool value - if true, Mime encoded message parts will be 
		decoded during parsing.
	  @return Message structure.
	 */
	public function parseMessage($decodeMime = true,$partID = 1,&$newMessage = false)
	{

    $ids = array();
		$ids['account_id'] = $this->folder->account->accountID;
		$ids['folder_id'] = $this->folder->name;
		$ids['item_id'] = $this->itemID;
		$ids['sid'] = session_id();
		
    $cache = Cache::instance($this->folder->account->user);


    $filename =  $cache->getMessageFile($cache->getMessageDir($ids['account_id'], $this->folder->folderID),$ids['item_id']);

    if(!file_exists($filename))
      $this->autoCreateMessage($filename);    
    
		//parse message
		$parser = new MailParse($filename, $ids, $decodeMime,'',$partID);

    $structure = $parser->parse();

    if($structure['info']['headers']) foreach($structure['info']['headers'] as $header => $value)
        if($value) 
          if(is_array($value))
            foreach($value as $hval)
              $structure['headers_plain'].=ucfirst($header).': '.$hval.CRLF;
          else
            $structure['headers_plain'].=ucfirst($header).': '.$value.CRLF;

		// if the message is parsed for the first time, generate its static flags
		// !!!!!!!!!!!! static flags has to be updated each time!
		//if (!($this->staticFlags & Item::ST_FLAG_CACHED) || $partID!='1') {
			$stFlags = self::getFlagsFromStructure($message, $structure, true,$filename);
			$this->setFullFlags($stFlags['flags'], 'staticFlags');
			
			$item = null;
			
			$item->hasAttachments = $stFlags['has_attachment'];
			$item->sMimeStatus = $stFlags['smime_status'];
			$item->from = $structure['headers']['from'];
			$item->to = $structure['headers']['to'];

			$item->priority = intval($structure['headers']['priority']);
			$item->subject = $structure['headers']['subject'];

      if(!$partID || $partID==1){
				$newMessage = true;
				$this->editProperties($item);
			}
		//}

		return $structure;
	}
	
	/**
	  @brief Get static flags of the item based on its message structure.
	  @param $message Message of the item.
	  @param $structure Parsed structure of the message
	  @param $cached If true, message has been cached already
	  @return Static flags of the item.
	 */
	private static function getFlagsFromStructure(&$message, $structure, $cached,$file = false)
	{
		$result = array();
		$result['flags'] = 0;
		
		if (MailParse::hasAttachments($structure)) 
			$result['has_attachment'] = true;
		else
			$result['has_attachment'] = false;
		
		if ($cached)
			$result['flags'] |= Item::ST_FLAG_CACHED;
		
		if (MailParse::hasHTMLBody($structure))
			$result['flags'] |= Item::ST_FLAG_HTMLBODY;
			
		$result['smime_status'] = MailParse::sMimeStatus($structure,$message,$file);
		
		return $result;
	}
	
	/**
	  @brief Get static flags of the item based on its message.
	  @param $message Message of the item.
	  @param $cached If true, message has been cached already
	  @return Static flags of the item.
	 */
	protected static function getStaticFlags(&$message, $cached = true,$file = false)
	{

		if($file)
		  $parser = new MailParse($file);
		else
		  $parser = new MailParse(false,false,false,false,'1',$message);
		  
		$structure = $parser->parse();
		return self::getFlagsFromStructure($message, $structure, $cached,$file);
	}
	/**
	  @brief Send ITEM data to output.
	 */
	public function sendData()
	{
    $file = $this->getMessageFile();
    if(!file_exists($file))
	   $this->autoCreateMessage($file);

    Tools::sendFileHeaders(self::getSubjectFileName('.eml'), filesize($file), 'message/rfc822');
	
    $fp = fopen($file,'rb');
      while ($buffer = fread($fp,65536))
        echo $buffer;
    fclose($fp);
	  
	}
	
	/**
	  @brief Send ITEM attachment data to output.
	  @param $partID ID of the attachment part
	 */
	public function sendAttachment($partID,$startID,$ids = array(),$sForceName = false)
	{
	  $file = $this->getMessageFile();
	  
	  if(!file_exists($file))
	   $this->autoCreateMessage($file);
	  
		$mailparse = new MailParse($file,$ids,true,'',$startID);		
	
    $mailparse->sendPart($partID,$sForceName);

	}
	

  public function getSubjectFileName($ext='') 
	{
	  // Get subject and date
	  $prefix=substr($this->subject,0,80);
	  $fdate=$item->date;

	  // Return new filename  
          return trim(str_replace(array('/','\\',':','*','?','"','|','<','>'), array(), $prefix . ($prefix?' - ':'') . date('YmdGis', $this->date) . $ext));
	}
	
	public function getAllAttachments()
	{
    $parser = new MailParse($this->getMessageFile());
    $structure = $parser->parse();
    $result = $structure['attachments'];
    if($result) foreach($result as $key => $attachment){
      $content = $parser->getPart($attachment['part_id'],$params);

      $result[$key]['file_content'] = $content;
    }

    return $result;
  }
  
  public function getAllAttachmentFiles($start_part_id = '1')
	{
    $parser = new MailParse($this->getMessageFile());
    $parser->startPart = $start_part_id;
    $structure = $parser->parse();
    $result = $structure['attachments'];
    if($result) foreach($result as $key => $attachment){
      $file = $parser->getPartFile($attachment['part_id'],$params);

      $result[$key]['file'] = $file;
    }

    return $result;
  }

// Universal functions for retrieving message content or message attachment to be used
// in webmailiqmessage.php as well as in download.php same functions have to be in gw/item.php

	public function getItemData(&$info = array())
	{
	  static $attachCount;
	  
	  $info['mimetype'] = 'message/rfc822';
    $info['name'] = 'message_'.((string) ++$attachCount).".eml";
    
    return $this->getMessage();
  }
  
	public function getAttachmentData($partID,&$info = array())
	{
		$file =  $this->getMessageFile();
		$mailparse = new MailParse($file);
		$attachment = $mailparse->getPart($partID,$params);
		$info['mimetype'] = $params['info']['content-type'];
		$info['name'] = Mailparse::getAttachmentName($params['info'],$partID);
		$info['encoding'] = $params['info']['transfer-encoding'];
		return $attachment;
	}
  
	public function getAttachmentDataFile($partID,&$info = array())
	{
		$file =  $this->getMessageFile();
		$mailparse = new MailParse($file);
		$attachmentFile = $mailparse->getPartFile($partID,$params);
		$info['mimetype'] = $params['info']['content-type'];
		$info['name'] = Mailparse::getAttachmentName($params['info'],$partID);
		$info['encoding'] = $params['info']['transfer-encoding'];
		return $attachmentFile;
	}
	

  public function initInvitation($partID,&$oIMIP)
  {
    $found = array();

    //Find the attachment with the text/calendar content type
    $sVersit = $this->getAttachmentData($partID,$info);

    //Return IMIP objects by reference
    $oIMIP = iMIP::load($this->folder->account->gwAccount);
    $oInvitation = $oIMIP->loadInvitation($sVersit);  

    return $oInvitation;
  }
  

  
  public function acceptInvitation($partID,$sFolder)
  {
    $oInvitation = $this->initInvitation($partID,$oIMIP);  
    $sReply = $oInvitation->accept($sFolder);

    $sReplyXML = $oIMIP->convertVersit($sReply);
    $aAttachment = $oIMIP->createAttachment($sReply);

    $aAttachment['xml'] = $sReplyXML;

    return $aAttachment;
    
  }

  public function declineInvitation($partID)
  {
    $oInvitation = $this->initInvitation($partID,$oIMIP);  

    $sReply = $oInvitation->decline();

    $aAttachment = $oIMIP->createAttachment($sReply);
    
    $aResult['xml'] = $oIMIP->convertVersit($sReply);

    return $aAttachment;
  }
  
  public function processInvitation($partID)
  {
    $oInvitation = $this->initInvitation($partID,$oIMIP);
    
    return $oInvitation->process();
  }
  

}

require_once('inc/imap/item.php');

?>