<?php

define('IMAP_ITEM_CHUNK_SIZE',100);
//TODO - remove dependance on model classes ( oFolder,oItem,cache - methods move,copy,deleteItems )
require_once('inc/item.php');

/**
  @class IMAP
  @brief Class for accessing IMAP accounts.
  @author Martin Zlomek
  @revision Pavel Perna
  @date 17-04-2007
 */
class IMAP {
	/** Resource representing an IMAP stream to the mailbox. */
	private $imap;

	/** Server part of the mailbox name. */
	private $server;

	/** Mailbox hierarchy delimiter. */
	public $delimiter;
	
	/** Array of objects of this class - only one instance should be available for each login details */
	public static $instance;
  private static $aOpenedMailboxes;
	/***********************/
	/* IMAP object methods */
	/***********************/

	/**
	  @brief Get instance of IMAP class
	  For each combination server/port/username/password there is an only one instance of this class allowed.
	  @param $server Name of the server where the mailbox is located.
	  @param $port Port where connect to on the server.
	  @param $username Name of the owner of the mailbox.
	  @param $password Password to the mailbox.
	  @return Instance of IMAP class
	 */
	public static function instance($account) 
  {
    $index = $account->accountID;
		if (!isset(self::$instance[$index])) {
        $imap = __CLASS__;
        self::$instance[$index] = new $imap($account->server, $account->port, $account->username, $account->password);
    }

    return self::$instance[$index];
  }
  
  /* Prevent users to clone the IMAP object */
  public function __clone()
  {
    throw new Exc('imap_clone');
  }

  public function msgNumber($uid)
  {
    return imap_msgno($this->imap,$uid);
  }
  
	
	/**
	  @brief Open an IMAP stream to a mailbox.
	  @param $server Name of the server where the mailbox is located.
	  @param $port Port where connect to on the server.
	  @param $username Name of the owner of the mailbox.
	  @param $password Password to the mailbox.
	 */
	private function __construct($server, $port, $username, $password)
	{
		// IMAP SSL
		if ($port == 993) $attr = '/ssl/novalidate-cert'; else
 		// POP3
                if ($port == '110') $attr ='/pop3/notls'; else
 		// POP3 SSL
                if ($port == '995') $attr ='/pop3/ssl/novalidate-cert'; else
	        // IMAP plain
                $attr = '/notls';
    
		$this->server = '{' . $server . ($port ? (':' . $port) : '') .  $attr . '}';

		$this->imap = @imap_open($this->server, $username, $password);
		
		if ($this->imap === false)
			self::error('imap_open');

		/* Get the mailbox hierarchy delimiter. */
		$list = imap_getmailboxes($this->imap, $this->server, 'INBOX');

    if (!is_array($list) || !$list)
			self::error('imap_delimiter_retrieval');
		
    $this->delimiter = $list[0]->delimiter;
		$this->accountID = $accountID;
    return true;

	}

	/**
	  @brief Close the IMAP stream.
	 */
	public function __destruct()
	{
		imap_close($this->imap, CL_EXPUNGE);
	  
  }

	/***********/
	/* Mailbox */
	/***********/

	/**
	  @brief Get the list of mailboxes.
	  @param $ownerID ID of owner account
	  @return The list as an array of objects.
	 */
	public function getMailboxes($ownerID = 'owner')
	{
		
    $list = imap_getmailboxes($this->imap, $this->server, '*');/*imap_getsubscribed*/
		
    if (!is_array($list))
			self::error('imap_getmailboxes');

		$mailboxes = array();
		foreach ($list as $mailbox) {
			$name = preg_replace(/*{*/ '/^{[^}]*}/', '', $mailbox->name);
      $acl  = @imap_getacl($this->imap, $name);

			if (isset($acl['owner'])) {
				$mailbox->rights = $acl['owner'];
			} else if (isset($acl[$ownerID])) {
				$mailbox->rights = $acl[$ownerID];
			} else {
				// This is temporary hack, because ACL doesn't work with public mailboxes.
				// Therefore a full access is signified and in case the user cannot actually
				// modify the folder, an error will be generated during modification.
				// TODO: this code should be removed once an IMAP PHP extending function
				// for getting proper access rights is written
				$mailbox->rights = 'ldwrsipc';
			}
			
			$mailbox->name	= $this->decode($name);
			$mailboxes[$mailbox->name] = $mailbox;
		}

		return $mailboxes;
	}

	/**
	  @brief Create a new mailbox.
	  @param $mailbox Name of the mailbox.
	  @param $ownerID ID of owner account
	  @return Rights of the new mailbox.
	 */
	public function createMailbox($mailbox, $ownerID = 'owner')
	{
		$mbox	= $this->encode($mailbox);
		$mailbox = $this->server . $mbox;

		if (!imap_createmailbox($this->imap, $mailbox))
			self::error('imap_mailbox_create',$mailbox);
		if (!imap_subscribe($this->imap, $mailbox))
			self::error('imap_mailbox_subscribe',$mailbox);

		$acl = @imap_getacl($this->imap, $mbox);

		if (isset($acl['owner'])) {
			$result = $acl['owner'];
		} else {
			$result = $acl[$ownerID];
		}
		//Store mailbox as opened
		$this->lastMailbox = $mailbox;
		
		return $result;
	}

	/**
	  @brief Delete a mailbox.
	  @param $mailbox Name of the mailbox.
	 */
	public function deleteMailbox($mailbox)
	{
		$mailbox = $this->server . $this->encode($mailbox);
		if (!imap_deletemailbox($this->imap, $mailbox))
			self::error('imap_mailbox_delete',$mailbox);
		if (!imap_unsubscribe($this->imap, $mailbox))
			self::error('imap_mailbox_unsubscribe',$mailbox);
	}

	/**
	  @brief Rename a mailbox.
	  @param $oldMailbox Name of the mailbox.
	  @param $newMailbox New name of the mailbox.
	 */
	public function renameMailbox($oldMailbox, $newMailbox)
	{
		$oldMailbox = $this->server . $this->encode($oldMailbox);
		$newMailbox = $this->server . $this->encode($newMailbox);
		
		if (!imap_renamemailbox($this->imap, $oldMailbox, $newMailbox))
			self::error('imap_mailbox_rename',$oldMailbox.'=>'.$newMailbox);
		if (!imap_unsubscribe($this->imap, $oldMailbox))
			self::error('imap_mailbox_unsubscribe_renamed',$oldMailbox);
		if (!imap_subscribe($this->imap, $newMailbox))
			self::error('imap_mailbox_subscribe_renamed',$newMailbox);
	}

	/**
	  @brief Delete multiple items from cache
	 */
	public function deleteItems($oFolder,$oItems = false,$sequence = false)
	{
	  	$result = true;
		$this->openMailbox($oFolder->name);

	    if ($oItems){
	      $sequence = array();
	      foreach($oItems as $oItem)
	          $sequence[] = $oItem->rid;
	      $sequence = implode(",",$sequence);
	  	}
	    if (!imap_setflag_full($this->imap, $sequence, '\\Deleted',ST_UID)
	  	|| !imap_expunge($this->imap)){
	  		self::error('imap_mailbox_delete_items');
	  	}
	}
	/**
	  @brief Move items from the IMAP folder to another one.
	  @param $from Name of the source mailbox.
	  @param $to Name of the destination mailbox.
	 */
	public function moveItems($from, $to, $items = false,$copy = false,$sequence = false)
	{
		$this->openMailbox($from->name);
		$nrMessages = imap_check($this->imap)->Nmsgs;
      
    $target = $this->encode($to->name);
    //Process copy/move bettween different accounts
    if($from->account->accountID!=$to->account->accountID){
	    //Check target
      if($to->itemClassName=='POP3Item')
	       throw new Exc($copy?'item_copy':'item_move');
      
      //Check target rights
      if(!($to->rights & Folder::RIGHT_INSERT  || $to->rights & Folder::RIGHT_D))
	       throw new Exc($bCopy?'item_copy_rights':'item_move_rights');	     
      
      //Empty folder - get all items
      if ($items===false)
        $items = $from->getItems();

      //Get & Create
    	foreach($items as $item){
  	    $fileName = $item->getMessageFile();
  	    
        if(!file_exists($fileName))
  	     $item->autoCreateMessage($fileName);

        $newID = $to->createItem(false,$fileName,true);

        if($to->itemClassName=='IMAPItem'){
          $strFlags = IMAPItem::encodeFlags($item->flags);
          //FIXME ? If this is not done the new message is not SEEN...
          $strFlags = str_replace("\\Flagged","",$strFlags);
          $targetIMAP = IMAP::instance($to->account);
          $targetIMAP->setFlags($to->name,$newID,$strFlags);
  		  }
  		  
        $aResult[$item->rid] = $newID;  
      }
      
      //Possibly delete
      if(!$copy) 
        $this->deleteItems($from,$items);
    //Process copy/move in the same account
    }else {
  		if ($items!==false){
    		foreach($items as $item)
    		  $sequence[] = $item->rid;
  			if(count($sequence)>IMAP_ITEM_CHUNK_SIZE){
		    	$chunks = chunk_split($sequence,IMAP_ITEM_CHUNK_SIZE);
		    	$parts = array();
				foreach($chunks as $part){
					$parts[] = implode(',',$part);
		  		}
	  		}else{
	  			$parts[0] = implode(',',$sequence);
	  		}
  		}else{
  			$parts = $this->splitSequence($sequence,$nrMessages);
  		}
  		$aResult = array();
	  	foreach($parts as $part){
	  		
	        if ($copy){
	          if (!($aPartResult = imap_mail_copy($this->imap, $part, $target,CP_UID)) 
	    			 || !imap_expunge($this->imap))
	            self::error('imap_mailbox_copyitems');
	  	    }else{
	          if (!($aPartResult = imap_mail_move($this->imap, $part, $target,CP_UID)) 
	    			 || !imap_expunge($this->imap))
	            self::error('imap_mailbox_moveitems');       
	        }
	        $aResult = $aResult + $aPartResult;
   		  }
   	 }
    foreach($aResult as $rid => $newRid){
    	$id = IMAP::fixID( $rid );
    	$newID = IMAP::fixID( $newRid );
    	unset($aResult[$rid]);
		$aResult[$id] = $newID;
    }
    return $aResult;
  }

	public function countMailboxItems($mailbox)
	{
	  $this->openMailbox($mailbox);
    return imap_check($this->imap)->Nmsgs;
  }

	/**
	  @brief Open an IMAP mailbox.
	  @param $mailbox Name of the mailbox.
	  @param $encoded Mailbox name encoded flag.
	 */
	private function openMailbox($mailbox, $encoded = false)
	{

    if (!$encoded)
			$mailbox = $this->server . $this->encode($mailbox);

    if ($this->lastMailbox == $mailbox)
			return;
		
		$this->lastMailbox = $mailbox;
		
		if (!imap_reopen($this->imap, $mailbox))
			self::error('imap_mailbox_open',$mailbox);
	}		
	
	/********/
	/* Item */
	/********/

	/**
	  @brief Get the list of items of the mailbox.
	  @param $mailbox Name of the mailbox.
	  @return The list as an array of objects.
	 */
	public function getItemsID($mailbox,$sequence = '1:*')
	{
    $this->openMailbox($mailbox);
        
		if ($num = imap_num_msg($this->imap)) {
		
			$list = $this->fetchFast($sequence);
		  
      if (!$list)
				self::error('imap_mailbox_message_list');

			return $list;
		}

		return array();
	}
	
  public function drop()
	{
	
  }
	
  public function getItems($mailbox,$sequence = '1:*',$fUID = 0)
	{
    $this->openMailbox($mailbox);
        
		if ($num = imap_num_msg($this->imap)) {
		
			$list = $this->fetchOverview($sequence,$fUID);

      if (!$list)
				self::error('imap_mailbox_message_list');

			return $list;
		}

		return array();
	}	
	
	/**
	  @brief Fetch overview of specified email messages
	  @param $sequence IMAP Sequence identifying the messages to fetch overview.
	  @return	Structure containing overview.
	 */
	private function fetchOverview($sequence,$fUID = 0)
	{	  
		return @imap_fetch_overview($this->imap, $sequence,$fUID);
	}
	
	/**
	  @brief Fetch basic info of specified email messages
	  @param $sequence IMAP Sequence identifying the messages to fetch info.
	  @return	Structure containing basic info.
	 */
	private function fetchFast($sequence)
	{
		return @imap_fetch_fast($this->imap, $sequence);    
	}
	
	/**
	  @brief Fetch message structure
	  @param $uid UID of the message.
	  @return	Structure containing message info.
	 */
	private function fetchStructure($uid)
	{
		return @imap_fetchstructure($this->imap, $uid, FT_UID);
  }

	/**
	  @brief Fetch message headers
	  @param $uid UID of the message.
	  @param $parse If TRUE, the headers will be parsed.
	  @return	Structure containing message headers.
	 */
	private function fetchHeaders($uid, $parse = false)
	{	
    $headers = @imap_fetchheader($this->imap, $uid, FT_UID);

    if ($parse) {
			
      $tmp = new MailParse(false,false,false,false,'1',$headers);
			$h =  $tmp->getHeaders();
      		  
      return $h;
		} else {
			return $headers;
		}
  }
  
  public function getAllHeaders()
  {
    return @imap_headers($this->imap);
  }
	
	/**
	  @brief Get details of specified message of the mailbox.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	  @param $getHeaders If TRUE, message headers will be included in result too.
	  @return The object containing detials of a message.
	 */
	public function getItemDetails($mailbox, $uid, $getHeaders = true, $item = false)
	{
    $this->openMailbox($mailbox);

    $sequence = @imap_msgno($this->imap, $uid);
        
    if (!$item) {
  		$list = $this->fetchFast($sequence);

  		if (!$list)
  			self::error('imap_message_details',$uid);
  		$result = reset($list);
    } else
      $result = $item;

		if ($getHeaders)
			$this->appendMessageHeaders($result);

		return $result;
	}
	
	/**
	  @brief Append message headers to message info object
	  @param $info Object with message info (obtained by fetchFast() function).
	  @param $headers Headers array of the message.
	 */
	private function appendMessageHeaders(&$info, $headers = array())
	{
		if (!$headers) {
			$uid = $info->uid;
			$headers = $this->fetchHeaders($uid, true);
		}
			
		// merge partial results	
		foreach ($headers as $key => $value) {
			$info->$key = $value;
		}
	}

	/***********/
	/* Message */
	/***********/

	/**
	  @brief Get a message.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	  @return Raw data of the message.
	 */
	public function getMessage($mailbox, $uid)
	{
		if ($mailbox)
		 $this->openMailbox($mailbox);

		$headers = $this->fetchHeaders($uid);

		if ($headers === false) 
			self::error('imap_no_headers',$uid);

    $body = @imap_body($this->imap, $uid, FT_UID);

		if ($body === false)
			self::error('imap_no_body',$uid);
		return $headers . $body;
	}

	/**
	  @brief Get a message.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	  @return Raw data of the message.
	 */
	public function saveMessageToFile($mailbox, $uid, $file)
	{
		if ($mailbox)
		 $this->openMailbox($mailbox);

		$headers = $this->fetchHeaders($uid);

		if ($headers === false) 
			self::error('imap_no_headers',$uid);

    file_put_contents($file,$headers);
    
      
    //MEMORY
    //imap_savebody($this->imap,$file, $uid,'', FT_UID);
    if(!imap_savebody_lowmemory($this->imap,$file, $uid))
      self::error('imap_no_body',$uid);

		return true;
	}
	
	/**
	  @brief Create a new message in a mailbox.
	  @param $mailbox The mailbox.
	  @param $message Raw data of the message.
	  @return The message item.
	 */
	public function createMessage($mailbox, $message, $file,&$newID = false)
	{
    $encoded_mailbox = $this->server . $this->encode($mailbox);

    if(!$file){

  		/* Create the message. */
  		if (!(@$result = imap_append($this->imap, $encoded_mailbox, $message)))
  			self::error('imap_create_message');

		  $parser = &new MailParse(false,false,false,false,'1',$message);
		  $headers = $parser->getHeaders(true);
    }else{


  		/* Create the message. */
  		if (!(@$result = imap_append_file($this->imap, $encoded_mailbox, $file)))
  			self::error('imap_message_create');  
  			
		  $parser = &new MailParse($file);
		  $headers = $parser->getHeaders(true);  		
    }

    
		if ($mailbox!="INBOX" && !@imap_reopen($this->imap, $encoded_mailbox))
			self::error('imap_mailbox_open',$encoded_mailbox);  

    $newID = IMAP::fixID($result);
    $result = @reset($this->fetchOverview($result,FT_UID));

		$this->appendMessageHeaders($result, $headers);

		return $result;
	}

	/**
	  @brief Delete a message.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	 */
	public function deleteMessage($mailbox, $uid)
	{
		$this->openMailbox($mailbox);

		if (!@imap_delete($this->imap, $uid, FT_UID) || !imap_expunge($this->imap))
			self::error('imap_message_delete',$uid);
	}

	/**
	  @brief Move a message to another mailbox.
	  @param $srcMailbox Name of the source mailbox.
	  @param $uid UID of the message.
	  @param $dstMailbox Name of the destination mailbox.
	  @param $copy If TRUE, the message will be copied instead of moved
	  @return New UID of the message.
	 */
	public function moveMessage($srcMailbox, $uid, $dstMailbox, $copy = false)
	{
		$this->openMailbox($srcMailbox);
		
		$mbox = $this->encode($dstMailbox);
		$mailbox = $this->server . $mbox;

		$msgID = $this->fetchOverview(imap_msgno($this->imap, $uid),FT_UID);
		$msgID = $msgID[0]->message_id;

		/* Move the message. */
		if ($copy)
		{
			if (!(@$result = imap_mail_copy($this->imap, $uid, $mbox, CP_UID)))
			{
				self::error('imap_message_copy',$uid);
			}
		} else {
			if (!(@$result = imap_mail_move($this->imap, $uid, $mbox, CP_UID))
				|| !@imap_expunge($this->imap))
			{
				self::error('imap_move_message',$uid);
			}
		}

		$this->openMailbox($mailbox,true);
		$result = reset($this->fetchOverview($result,FT_UID));

		return $result;
	}

	/**
	  @brief Set flags on the message.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	  @param $flags The flags.
	 */
	public function setFlags($mailbox, $uid, $flags)
	{
		$this->openMailbox($mailbox);

		if (!@imap_setflag_full($this->imap, $uid, $flags, ST_UID))
			self::error('imap_flags_set',$uid);
	}

	/**
	  @brief Unset flags on the message.
	  @param $mailbox Name of the mailbox.
	  @param $uid UID of the message.
	  @param $flags The flags.
	 */
	public function unsetFlags($mailbox, $uid, $flags)
	{
		$this->openMailbox($mailbox);

		if (!@imap_clearflag_full($this->imap, $uid, $flags, ST_UID))
			self::error('imap_flags_clear',$uid);
	}
	

	
	public function search($mailbox, $sPhrase){
    $this->openMailbox($mailbox);
    $sPhrase = 'ALL TEXT "'.$sPhrase.'"';
    
    return imap_search($this->imap, $sPhrase, SE_UID);

  }
	
	
	/**
	  @brief Get just right created message in a mailbox.
	  @param $sequence Posible interval where the message could be.
	  @param $message Raw data of the message.
	  @param $getHeaders If true, message headers will be included in the result.
	  @return The message item.
	 */
	private function getNewMessage($msgID, $mailbox)
	{
		/* Get all possible items. */
		$new = imap_search($this->imap, 'RECENT');

    if (!$new)
      $new = imap_search($this->imap,'UNFLAGGED');
			
      
    if(!$new)
       self::error('imap_message_not_found');
	
		/*$sequence = '';
		foreach ($new as $item)
			$sequence .= ($sequence?',':'') . $item;
		*/
		
		$sequence = $new[0].":*";
		
		$candidates = $this->fetchOverview($sequence);
		if($msgID){
      if (count($candidates) == 1) {
  		  $item = reset($candidates);
        if($item->message_id == $msgID)
  			 return $item;
  		} else {
  			foreach ($candidates as $item) {
  				if ($item->message_id == $msgID)
  					return $item;
  			}
  		}
		}
		// Message not found:
		self::error('imap_message_not_found');
	}

	/****************/
	/* Tool methods */
	/****************/

	/**
	  @brief Convert mailbox name to IMAP format.
	  @param $mailbox Name of the mailbox.
	  @return Converted mailbox name.
	 */
	private function encode($mailbox)
  {
    if ($this->delimiter != '/')
    $mailbox = str_replace('/', $this->delimiter, $mailbox);
    
    return mb_convert_encoding($mailbox, 'UTF7-IMAP', 'UTF-8'); 
  }

  /**
  @brief Convert mailbox name from IMAP format.
  @param $mailbox Name of the mailbox.
  @return Converted mailbox name.
  */
  private function decode($mailbox)
  {
    $mailbox = mb_convert_encoding($mailbox, 'UTF-8', 'UTF7-IMAP');
    if ($this->delimiter != '/')
    $mailbox = str_replace($this->delimiter, '/', $mailbox);  
    return $mailbox;
  }
  
  public function getBody($mailbox,$itemID){
    $this->openMailbox($mailbox);
    return substr(imap_body($this->imap,$itemID,FT_UID|FT_PEEK),0,DB_BODY_LIMIT);     
  }
	
	/**
	  @brief Throw an Exc exception with the error message.
	  @param $message If there is no error message from the IMAP driver,
	  use this one.
	  @throw Exc exception with the error message.
	 */
	private static function error($id,$message = '')
	{
		$errors = imap_errors();
		if ($errors)
			throw new Exc('imap_internal',$errors[0]);
		else 
			throw new Exc($id,$message);
	}
		
	static public function fixID( $id )
	{
		return str_pad($id, 10, '0', STR_PAD_LEFT);
	}
	
		
	public function splitSemicolon($str,$nrMessages)
	{
		$str = explode(':',$str);
		if ($str[1]=='*'){
			$str[1] = intval($nrMessages);
		}
		$count = intval($str[1]) - intval($str[0]) + 1;
		if($count > IMAP_ITEM_CHUNK_SIZE){
			$lower = intval($str[0]);
			$upper = intval($str[1]);
			$chunkCount = ceil($count/IMAP_ITEM_CHUNK_SIZE);
			for ($i = 1; $i <= $chunkCount;$i++){
				if ($i==$chunkCount){
					$upper = intval($str[1]);
				}else{
					$upper = intval($lower) + IMAP_ITEM_CHUNK_SIZE;
				}
				$range[] = $lower.':'.$upper;
				$lower+=IMAP_ITEM_CHUNK_SIZE;
			}
		}else{
			$range[] = intval($str[0]).":".intval($str[1]);
		}
		return $range;
	}
	
	public function splitSequence($str,$nrMessages)
	{
		$range = array('sequences'=>array(),'ids'=>array());
		$str = explode(',',$str);
		foreach($str as $part){
			if(strpos($part,':')){
				$range['sequences'] = array_merge(
					$range['sequences'],
					self::splitSemicolon($part,$nrMessages)
				);
			}else{
				$range['ids'][] = $part;
			}
		}
		$result = $range['sequences'];
		if(count($range['ids'])>IMAP_ITEM_CHUNK_SIZE){
			$ids = chunk_split($range['ids']);
			foreach($ids as $part){
				$result[] =implode(',',$part);
			}
		}
		return $result;
	}

}

?>
