<?php

/**
  @class Item
  @brief Class representing a generic item.
  @author Martin Zlomek
  @date 09-02-2006
 */
abstract class Item {
	/** List of item flags. */
	/** { */
	const FLAG_ANSWERED = 0x01; /**< Message has been answered. */
	const FLAG_DELETED  = 0x02; /**< Message is "deleted" for removal by later
								  EXPUNGE. */
	const FLAG_DRAFT	= 0x04; /**< Message has not completed composition
								  (marked as a draft). */
	const FLAG_FLAGGED  = 0x08; /**< Message is "flagged" for urgent/special
								  attention. */
	const FLAG_RECENT	= 0x10; /**< Message is "recently" arrived in this
								  mailbox. */
	const FLAG_SEEN	 = 0x20; /**< Message has been read. */
	
	const FLAG_FORWARDED = 0x40; /**< Message has been forwarded. */
	/** } */
	
	/** List of item static flags. */
	/** { */
	const ST_FLAG_HTMLBODY = 0x01; /**< Message has HTML body. */
	const ST_FLAG_CACHED = 0x02; /**< Message has been stored to cache already. */
	/** } */
	
	/** Item priority. */
	/** { */
	const NORMAL_PRIORITY = 3; /**< Value of normal priority of a message. */
	/** } */
	
	/** Size of the item. */
	public $size;

	/** Date of the item. */
	public $date;

	/** Flags of the item. */
	public $flags;

	/** From header of the item. */
	public $from;

	/** To header of the item. */
	public $to;

	/** Subject of the item. */
	public $subject;
	
	/** Static flags of the item (cannot be set by webmail client). */
	public $staticFlags;
	
	/** Priority of the item. */
	public $priority;
	
	/** Color of the item. */
	public $color;

	/**
	  @brief Create a new item object.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	protected function __construct($size, $date, $flags, $from, $to, $subject, 
				$staticFlags, $priority, $color, $sMimeStatus, $hasAttachments)
	{
		$this->size	= $size;
		$this->date	= $date;
		$this->from	= $from;
		$this->to	  = $to;
		$this->priority = $priority;
		$this->subject  = $subject;
		$this->flags    = $flags;
		$this->color    = $color; 
		$this->staticFlags = $staticFlags;
		$this->sMimeStatus = $sMimeStatus;
		$this->hasAttachments = $hasAttachments;
		
	}

	/**
	  @brief Create a new item.
	  @param $folder Folder object of the item.
	  @param $message The message of the item.
	 */
	protected static function create($folder, $message)
	{
	}

	/**
	  @brief Delete the item.
	 */
	abstract protected function delete();

	/**
	  @brief Create a new item.
	  @param $folder Destination folder object.
	 */
	abstract protected function move($folder);

	/**
	  @brief Set flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function setFlags($flags, $field)
	{
		$this->$field |= $flags;
	}

	/**
	  @brief Unset flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function unsetFlags($flags, $field)
	{
		$this->$field &= ~$flags;
	}
	
	/**
	  @brief Check whether new flags are necessary to be set.
	  @param $flags New flags.
	  @param $field Identificator of field where flags are stored.
	  @return True if new flags differ from already set flags, False otherwise.
	 */
	protected function checkSetFlags($flags, $field)
	{
		$curFlags = intval($this->$field);
		$flags = intval($flags);
		$set = $curFlags & $flags;
		if ($set === $flags)
			return false;
		else
			return true;
	}
	
	/**
	  @brief Check whether new flags are necessary to be unset.
	  @param $flags New flags.
	  @param $field Identificator of field where flags are stored.
	  @return False if new flags differ from already set flags, True otherwise.
	 */
	protected function checkUnsetFlags($flags, $field)
	{
		$curFlags = intval($this->$field);
		$flags = intval($flags);
		$set = $curFlags & $flags;
		if ($set === 0) 
			return false;
		else
			return true;
	}
	
	/**
	  @brief Set flags of the item to the specified value.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function setFullFlags($flags, $field = 'flags')
	{
		$curFlags = intval($this->$field);
		$flags = intval($flags);
		$unsetFlags = $curFlags & ~$flags;
		$setFlags = $flags & ~$curFlags;
		

		if ($setFlags)
			$this->setFlags($setFlags, $field);
		if ($unsetFlags){
			$this->unsetFlags($unsetFlags, $field);
    }
  }
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	abstract protected function getMessage();
}

require_once('inc/cache/item.php');

?>
