<?php

require_once('inc/cache/folder.php');
require_once('inc/localpop/localpop.php');
require_once('inc/localpop/item.php');

/**
  @class LocalPOPFolder
  @brief Class representing a LocalPOP folder.
  @author Pavel Perna
  @date 17-04-2007
 */
class LocalPOPFolder extends CacheFolder {

	//Type of the folder == M for mail items 
	public $type;  
	public $lpop;
	
	/**
	  @brief Create a new LocalPOP folder object.
	  @param $account Parent account object of the folder.
	  @param $folderID ID of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */	 
	public function __construct(&$account, $folderID, $name, $rights = 255, $sync)
	{
		parent::__construct($account, $folderID, $name, 255, $sync);
		$this->type = 'M';
    $this->lPOP = true;
		$this->itemClassName = 'LocalPOPItem';
	}
	
	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	  @return IMAPFolder object.
	 */
	public static function create($account, $name)
	{

    /* Create the folder on the IMAP server. */
		$lpop = LocalPOP::instance($account);
    
    /* Create the folder in filesystem */
    $lpop->createMailbox($name);
    
		/* Create the folder in the folder cache. */
		$folderID = parent::create($account, $name, 255);

		return new LocalPOPFolder($account, $folderID, $name, 255, false);
	}
	
	/**
	  @brief Delete the folder.
	 */
	public function delete()
	{
		$account = $this->account;
		
		//Get parent folder
		$oFolderParent = $this->getParent($account);

		$lpop = LocalPOP::instance($account);

		$folders = krsort($account->folders["main"]);

		/* Delete all sub-folders from the LPOP  */
		foreach ($account->folders["main"] as $folder) {

			if (strpos($folder->name, $this->name . '/') !== 0)
				continue;
      
			@$lpop->deleteMailbox($folder->name);

			unset($account->folders['main'][$folder->name]);
		}   

		/* Delete the folder itself from the IMAP server. */
		@$lpop->deleteMailbox($this->name);
		
		/* Delete the folder including sub-folders from the folder cache. */
		parent::delete();
		
		/* Delete the folder in the list of folders */
		unset($account->folders['main'][$this->name]);
    
		/* Delete groupware virtual folders too - FIXME move to ImapFolder->delete*/
		$oGWFolder = $this->getGWFolder($this->name); 
		if(!($oGWFolder===false)){
			if ($oGWFolder->type=='M') $oGWFolder->delete();
		}
  	
		// Delete parent IMAP virtual folders
		if ($oFolderParent){     
			//Are there any GW parent folders with the same name as parent folder
			//(is the parent folder virtual)
			$oParentGWFolder = $this->getGWFolder($oFolderParent->name);
			if (!($oParentGWFolder === false))
				$bDeleteFlag = true;
			else  $bDeleteFlag = false;
          
		//Get Childern of the parent IMAP folder
		$oChildern = $oFolderParent->getChildern($account);   
          
		//If no IMAP childern and Delete flag(virtual IMAP folder) 
		if ($oChildern===false && $deleteFlag){
			$oGWFolder = $this->getGWFolder($oFolderParent->name);
			if ($oGWFolder->type!='M') $oFolderParent->delete();
		}
	}	 

	}	
	
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	public function rename($newName)
	{

		$account = $this->account;

		$lpop = LocalPOP::instance($account);

		$length = strlen($this->name);

    $type = 'M';
    $parent = substr($newName,0,strrpos($newName,$lpop->delimiter));

    
    //Check for parent folder
    if($parent){
      try{
        $oFolder = $account->getFolder($parent,$type);
      }catch(Exc $e){
        $lpop->createMailbox($parent);   
      }
    }

		/* Rename the folder itself on the IMAP server. */
		$lpop->renameMailbox($this->name, $newName);

		/* Rename the folder including sub-folders in the folder cache. */
		parent::rename($newName);
	}

	public function deleteItems($oItems = false)
	{
    $oAccount = $this->account;
    
    //Create LocalPOP instance
    $oLPOP = LocalPOP::instance($oAccount);

    //Delete from server
    $oLPOP->deleteItems($this,$oItems);
    
    //Delete from cache
    parent::deleteItems($oItems);
  }

	public function moveItems($oFolder,$oItems = false)
	{
		$account = $this->account;

		$lpop = LocalPOP::instance($account);
		$aMovedID = $lpop->moveItems($this, $oFolder, $oItems);

		// Move all items from the folder in the folder cache.
   		parent::moveItems($oFolder,$oItems,$aMovedID);
		
	}
	public function copyItems($oFolder,$oItems = false)
	{
		$account = $this->account;

		$lpop = LocalPOP::instance($account);
		$aCopiedID = $lpop->moveItems($this, $oFolder, $oItems,true);
		
		// Copy all items from the folder in the folder cache.
		parent::copyItems($oFolder,$oItems,$aCopiedID);
	}  	
	/**
	  @brief Get the list of folder items.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of IMAPItem objects.
	 */
	public function getItems($aFilterTag = array())
	{
		$filter = $aFilterTag['sql'];
		$orderBy = $aFilterTag['orderby'];
		$offset = $aFilterTag['offset'];
		$limit = $aFilterTag['limit'];
		$search =  $aFilterTag['fulltext'];
		$fields = $aFilterTag['tag'];

		// always get RID attribute of message
		//if ($fields && !strstr($fields, 'rid'))
		//	$fields .= ',rid';

		return parent::getItems('LocalPOPItem', $filter, $orderBy, $offset, $limit,$fields,$search);
	}
	
	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return IMAPItem object.
	 */
	public function getItem($itemID,$cache = array())
	{
		return parent::getItem('LocalPOPItem', $itemID, $cache);
	}  
	
	/**
	  @brief Create a new item.
	  @param $message The message of the item.
	 */
	public function createItem($message, $file,$bNoCache = false)
	{
		return LocalPOPItem::create($this, $message, $file, $bNoCache);
	}	

	/**
	  @brief Synchronize the item cache with the IMAP server.
	 */
	public function sync()
	{		
    if($_SESSION['DEBUG'])log_buffer("SYNCHRONIZE FOLDER : $this->name \r\n==========================================\r\n",'DEBUG');
		$account = $this->account;

		$lpop = LocalPOP::instance($account);

		$aItems = $lpop->getItemsID($this->name);

		//Create temporarily unavailable
		@file_put_contents($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE,1);
		$oldExecutionTime = ini_alter('max_execution_time',MAX_EXECUTION_TIME);    

		parent::sync($this->itemClassName,$aItems,$this);

		//Delete temporarily unavailable 
		@unlink($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE);
		ini_set('max_execution_time',$oldExecutionTime);        

	}  	
	
  public function search($sPhrase,$aFilterTag){
    if($_SESSION['FULLTEXT'])
      return parent::search($sPhrase,$aFilterTag);   
    else{
      $lpop = LocalPOP::instance($this->account);
      $result = $lpop->search($this,$sPhrase,$aPaths);
    }
      
    $aFilterTag = $this->updateSearchFilter($result,$aFilterTag);
    $result = $this->getItems($aFilterTag);
    
    return $result;
  }
}

?>
