<?php
require_once ('inc/cache/item.php');
/**
  @class LocalPOPItem
  @brief Class representing a LocalPOP item.
  @author Pavel Perna
  @date 09-02-2006
 */
class LocalPOPItem extends CacheItem {
	/**
	  @brief Create a new IMAP item object.
	  @param $folder Parent folder object of the item.
	  @param $itemID ID of the item.
	  @param $rid Remote ID of the item.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	public function __construct($folder, $itemID, $rid,$size, $date, $flags, 
								$from, $to, $subject, $staticFlags, $priority, $color,
								$sMimeStatus, $hasAttachments)
	{

  		parent::__construct($folder, $itemID, $rid, $size, $date, $flags, 
							$from, $to, $subject, $staticFlags, $priority, $color,
							$sMimeStatus, $hasAttachments);
      $this->wmclass = 'M';
	}
	
	/**
	  @brief Decode an item from IMAP format.
	  @param[in,out] $item The IMAP item.
	  */
	public static function decode(&$item,$staticFlags = false,$folder = false,$protocolHandler = false)
	{
		// LocalPOP item doesn't need to be decoded, this functionality is already
		// done by LocalPOP::createMessage() method instead of date
		if (!is_int($item->date))
		  $item->date = strtotime($item->date);

    
    //Read the date from the file creation time
    if(!$item->date || $item->date == -1){

      $sDir = $protocolHandler->mailboxPath;
      
      $sName = strtolower($folder->name)=='inbox'?$sDir:(($sDir.$folder->name).'/');
      
      $sFilename = $sName.$item->uid.localpopmessageext;
      @$item->date = filectime($sFilename);
    }
 	}	
	
		/**
	  @brief Create a new item.
	  @param $folder Parent folder object of the item.
	  @param $message The message of the item.
	  @param $file The message file of the item.
	  @param $bNoCahce Not create message in cache
	 */
	public static function create($folder, $message,$file = false,$bNoCache = false)
	{
		$account = $folder->account;

		/* Create the item on the IMAP server. */
		$lpop = LocalPOP::instance($account);
		$item = $lpop->createMessage($folder->name, $message, $file, $newID);	
		
    if($bNoCache)
		  $return = $newID;
		else
		/* Create the item in the item cache. */
		 $return =  parent::create($folder, $item, $message, array(),$file);
		 
		return $return;
	}

	/**
	  @brief Delete the item.
	 */
	public function delete($cache = array())
	{
		$folder  = $this->folder;
		$account = $folder->account;
    
		/* Delete the item from the IMAP server. */
		$lpop = LocalPOP::instance($account);
		$lpop->deleteMessage($folder->name, $this->rid);

		/* Delete the item from the item cache. */
		parent::delete($cache);
	}
	
	/**
	  @brief Edit the message of the item.
	  @param $message New message of the item.
	 */
	public function edit($message,$file = false)
	{
		$folder = $this->folder;
		$account = $folder->account;

		/* Delete the original message on IMAP server and create a new one. */
		$lpop = LocalPOP::instance($account);
		$item = $lpop->createMessage($folder->name, $message, $file);
		$lpop->deleteMessage($folder->name, $this->rid);

		/* Edit the item in the item cache. */
		return parent::edit($item, $message, array(), $file);
	}
	
	/**
	  @brief Move an item to another folder.
	  @param $folder Destination folder object.
	 */
	public function move($folder, $cache = false)
	{
		$account = $folder->account;

		/* Move the item to another folder on the IMAP server. */
		$lpop = LocalPOP::instance($account);
		$this->rid = $lpop->moveMessage($this->folder->name, $this->rid,
										$folder->name);
		/* Move the item to another folder in the item cache. */
		parent::move($folder, $cache);
	}	
	
	/**
	  @brief Copy an item to another folder.
	  @param $folder Destination folder object.
	 */
	public function copy($folder, $cache = false)
	{
	  $account = $folder->account;

		/* Get original message */
		  $fCahce = $cache;
			if($cache === false)
       $cache = Cache::instance($account->user);


		//$message = $cache->getMessage($account->accountID, $this->folder->folderID, $this->rid);
		
		/* Move the item to another folder on the IMAP server. */
		$lpop = LocalPOP::instance($account);
		
		
		$this->rid = $lpop->moveMessage($this->folder->name, $this->rid,
										$folder->name, true);

		/* Move the item to another folder in the item cache. */
		parent::copy($folder, $message, $cache);
	}	
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function getMessage()
	{
		$folder  = $this->folder;
		$account = $folder->account;
    
    
		/* Get cached message. */
		$message = parent::getMessage();

		if ($message === false) {
			/* Get IMAP message. */
			$lpop = LocalPOP::instance($account);
			$message = $lpop->getMessage($folder->name, $this->rid);

			/* Cache the message. */
			parent::createMessage($message);
		} 
    
    else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		
		// remove CRLF.CRLF or LF.LF from the end of the message
		if (substr($message, -5) == "\r\n.\r\n") {
			$message = substr($message, 0, -5);
		} else if (substr($message, -3) == "\n.\n") {
			$message = substr($message, 0, -3);
		}
		
		return $message;
	}
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function autoCreateMessage($filename)
	{
		$folder  = $this->folder;
		$account = $folder->account;

		/* Check cached message. */
		if (parent::getMessage() === false) {

			/* Get IMAP message. */
			$lpop = LocalPOP::instance($account);
			
			$lpop->saveMessageToFile($folder->name, $this->rid, $filename);
			Tools::checkMessageEnd($filename);

		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return true;
	}
  		
	public function markAsRead()
	{
		$this->setFlags(Item::FLAG_SEEN);
	}
 	public function markAsUnread()
	{
		$this->unsetFlags(Item::FLAG_SEEN);
	} 	
	/**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function setFlags($flags, $field = 'flags')
	{
		// if all required flags are set already, do nothing
		if (!$this->checkSetFlags($flags, $field))
			return;
			
		/* Set flags on the cached item. */
		parent::setFlags($flags, $field);
	}

	/**
	  @brief Unset specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function unsetFlags($flags, $field = 'flags')
	{
		// if all required flags are unset already, do nothing
		if (!$this->checkUnsetFlags($flags, $field))
			return;

		/* Unset flags on the cached item. */
		parent::unsetFlags($flags, $field);
	}
	
}
?>