<?php
require_once('inc/encode.php');
require_once('inc/phpmailer/class.phpmailer.php');
require_once('inc/smime.php');
require_once('inc/imip.php');



/**
 * @class MailParse
 * @brief Class for parsing rfc822 and rfc2045 email messages.
 * @author Martin Zlomek
 * @revision Pavel Perna
 * @date 17-04-2007	
 */
class MailParse {
	/** Resource representing the mail. */
	public $mail;
	
	/** This variable is used as a storage of decoded SMime message, if the message
	 file consists of the only one SMime decrypted part	*/
	private $sMimeMessage;

	/** The message. */
	private $message;

	/** Parts of the message. */
	private $parts;

	/** IDs of the message. */
	private $ids;
	
	/**
		@brief Create the parser for a message.
		@param $message The message.
		@param $ids Array of all IDs of the message item.
		@param $decodeMime Bool value - if true, Mime encoded message parts will be 
		decoded during parsing.
		@param $attIdPrefix Prefix for the attachmend IDs. Do not set this variable,
		it is used only when calling this class recursively for SMime encoded parts
		from $this->decryptPart() method.
	 */
	public function __construct($file = '',$ids = array(), $decodeMime = false, $attIdPrefix = '',$partID = '1',$fromString = '')
	{
		//Construct from file or string
		if(!$file){
			if(!$fromString)
				throw new Exc('mailparse_empty');
			else{
				$this->mail = mailparse_msg_create();
				mailparse_msg_parse($this->mail,$fromString);
				$this->message = $fromString;
			}
		} else{
			if(!@$this->mail = mailparse_msg_parse_file($file))
				throw new Exc('mailparse_parser_init');
			$this->file = $file;
		}
		
		$this->originalParts = mailparse_msg_get_structure($this->mail);

		//Create tree based array to store parts structure 
		//and make pointers
		if ($this->originalParts) foreach($this->originalParts as $part){
			$cursor = &$this->parts;
			$hierarchy = explode(".",$part);
			
			if(is_array($hierarchy)) foreach($hierarchy as $id){
				if(!isset($cursor[$id])) $cursor[$id] = array();
				$cursor = &$cursor[$id];
			}
			$this->partPointer[$part] = &$cursor;
		}
		$this->startPart = $partID;

		$this->ids = $ids;

		$this->decodeMime = $decodeMime;
		$this->attIdPrefix = $attIdPrefix;		
		if(!$this->startPart) $this->startPart = 1;
	}
	
	public function __destruct()
	{
		mailparse_msg_free($this->mail);
	}
	
	public function parse()
	{
		
		if(($pos = strpos($this->startPart,'1.decrypt.'))!==false){
			$this->decryptedPart = str_replace('1.decrypt.','',$this->startPart);
			$this->startPart = '1';
		}
		
		$this->parsePart($this->startPart);

		if($this->partIDChange)
			$this->startPart = $this->partIDChange;

		
		
		
		//Return requested part
		$this->aResult = $this->aParts[$this->startPart];

		//Add uuencoded attachments to the message
		if($this->aUUAttachments)
			foreach($this->aUUAttachments as $upart => $attachment){
				$this->aResult['attachments'][] = $attachment[0];
			}
		$this->replaceCID();
		
		if ($this->decodeMime) {
			$this->checkSMimeParts($this->aResult);

			if ($this->sMimeMessage){
			 $this->sMimeMessage['encrypted'] = true;
			 return $this->sMimeMessage;
			}
		}
		return $this->aResult;
	}

	/**
	 * MailParse::htmlExists()
	 * 
	 * Check if there is some html part in above parts
	 * 
	 * @param mixed $parent
	 * @return
	 */
	private function htmlExists($parent)
	{
		$cursor = $parent;
		while(isset($this->aParts[$cursor]) && strpos($cursor,'.')!==false){
			if($this->aParts[$cursor]['html_body']){
				return true;
			}
			$cursor = substr($cursor,0,strrpos($cursor,'.'));
		}
		if($this->aParts[$cursor]['html_body']){
			return true;
		}
		return false;
	}
	private function parsePart($partID)
	{
		$subPart = 0;
				
		$part = mailparse_msg_get_part($this->mail, $partID);
		$info = mailparse_msg_get_part_data($part);

		//Special handling for message/rfc822

		if($info['content-type']=='message/rfc822' && $partID==$this->startPart){
			
			$partID.='.1';
			$this->partIDChange = $partID;
			$this->rfc822 = true;
			$part = mailparse_msg_get_part($this->mail, $partID);
			$info = mailparse_msg_get_part_data($part);
		}

		//Parse headers
		$this->aParts[$partID]['headers'] = self::parseHeaders($info);
		$this->aParts[$partID]['info'] = $info;
		if (strpos($info['content-type'], 'multipart/') === 0 ){
			$this->aParts[$partID]['multipart']  = true;
			
			if(strpos($info['content-type'], 'report') !== false){
				$this->aParts[$partID]['report'] = true;
			}
			if(strpos($info['content-type'], 'mixed') !== false){
				$this->aParts[$partID]['mixed'] = true;
			}
			if(strpos($info['content-type'], 'alternative') !== false){
				$this->aParts[$partID]['alternative'] = true;
			}
			while ($subPart++ < count($this->partPointer[$partID])){
				$this->parsePart($partID.'.'.($subPart));
			}
		}
		//Parent part if no subparts, == partID
		$parent = explode(".",$partID);
		if(count($parent)>1) unset($parent[count($parent)-1]);
		$parent = implode(".",$parent);

		//Behave depending on content type
		switch($info['content-type'] ){
			/************************************************************************
			 *														SINGLE PART																 *
			 *																																			*
			 ************************************************************************
			 */		 
			 //Plain text part 		
			case 'text':
			case 'text/plain':
				$body = $this->parseBody($part, $info, $partID);
				if($info['content-disposition']=='inline'){
					$this->aParts[$parent]['plain_body'].="-----------------\r\n".$body;
					//Check if exists some html to append to 
						
					
					if($this->htmlExists($parent)){
						$this->aParts[$parent]['html_body'].='<hr/>'.Tools::text2html($body,true);
					}
				}
				//As attachment ?
				if (isset($info['content-name']) || isset($info['headers']['content-name']) || $info['content-disposition']=='attachment'){
					$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
				//As text ?
				}else{
					if(!isset($this->aParts[$parent])){
						$parent.='.1';
					}
					if($this->aParts[$parent]['mixed']){
						$this->aParts[$parent]['plain_body'] .= $body;
					}else{
						$this->aParts[$parent]['plain_body'] = $body;
					}
					$this->aParts[$parent]['plain_body_info'] = $info;
				}
			break;
			 //HTML part
			case 'text/html':
					$body = $this->parseBody($part, $info, $partID);
					if($info['content-disposition']=='inline'){
						$this->aParts[$parent]['plain_body'].="-----------------\r\n".Tools::removeHTML($body);	
					if($this->htmlExists($parent)){
						$this->aParts[$parent]['html_body'].='<hr/>'.$body;
					}
				}
					
				//As attachment ?
				if (isset($info['content-name']) || isset($info['headers']['content-name']) || $info['content-disposition']=='attachment'){
					$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
				//As html ?		
				} else{
					if($this->aParts[$parent]['mixed']){
				 		$this->aParts[$parent]['html_body'] .= $body;
				 	}else{
						$this->aParts[$parent]['html_body'] = $body;
					}
				}
				$this->aParts[$parent]['html_body_info'] = $info;
				break;
			case 'text/calendar':

				$aAttachment = $this->parseAttachment($part, $info, $partID);
				$oGWAccount = &$_SESSION['user']->aAccounts[$_SESSION['EMAIL']]->gwAccount;
				$oIMIP = iMIP::load($oGWAccount);
				
				$aAttachment['imip_raw'] = $this->parseBody($part, $info, $parent);
				$aAttachment['imip_xmlstr'] = $oIMIP->convertVersit($aAttachment['imip_raw']);
				$aAttachment['imip_xml'] = simplexml_load_string($aAttachment['imip_xmlstr']);

				//preg_match("#METHOD:([^\r^\n]{0,})#i",$aAttachment['imip_raw'],$aMatch);
				//$sMethod = strtoupper($aMatch[1]);
				
				//Process vCalendar automaticaly if $info['content-method'] = 'REPLY|CANCEL'
		//	print_r($info['content-method']);die();
				switch(strtoupper($info['content-method'])){
					case 'REPLY':
					case 'CANCEL':
						$oInvitation = $oIMIP->loadInvitation($aAttachment['imip_raw']);
						$oInvitation->process();
					break;
				}
				$this->aParts[$parent]['attachments'][] = $aAttachment;

			break;
				
			//RFC 822 messages
			
			case 'message/rfc822':
				if(/*!$info['headers'] && */$partID!=$this->startPart){
					//$partID .='.1';
					//$part = mailparse_msg_get_part($this->mail, $partID);
					//$info = mailparse_msg_get_part_data($part);
					
					$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
					break;
				}
				//Set as attachment - ignore inline
				if (isset($info['content-name']) || $info['content-disposition']=='attachment' || $info['content-disposition']=='inline'){
					$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
				//As body
				}else{
					//? isset html_body
					if(!$this->htmlExists($parent)){
						//? $partID
						$this->aParts[$parent]['html_body'] = Tools::text2html($this->aParts[$partID]['plain_body']);
					}
					//? $partID
					$this->aParts[$parent]['plain_body'] .= $this->aParts[$partID]['plain_body'];
				}
				break;
			case 'message/delivery-status':
					$this->aParts[$parent]['delivery-status'] = $this->parseBody($part, $info, $partID);
			break;
			case 'application/ms-tnef':
				$this->parseTnefPart($part,$parent,$partID);
				break;
			case 'application/x-pkcs7-signature':
			case 'application/pkcs7-signature':
				 /* if(!isset($this->aParts[$parent])){
						$parent.='.1';
					} */
					
				$this->aParts[$parent]['signatures'][] = $this->parseAttachment($part, $info, $partID);
				break;
			case 'application/x-pkcs7-mime':
			case 'application/pkcs7-mime':
				if(!isset($this->aParts[$parent])){
					$parent.='.1';
				} 
				$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
			break;
			case 'application/octet-stream':
				if(strtolower($info['content-name'])==='winmail.dat'){
					$this->parseTnefPart($part,$parent,$partID);
					break;
				}
			//All other mime-types as an attachment
			case 'text/enriched':
			default:
					if(!isset($this->aParts[$parent])){
						$parent.='.1';
					}
					//As an attachment
					$attachment = $this->parseAttachment($part, $info, $partID);
					//Insert inline images
					if(
						strpos($info['content-type'],'image/')!==false
						&& $attachment['cid']===''
						&& $info['content-disposition'] == 'inline'
					){
						$attachment['cid'] = '_webmail_'.uniqid(rand(time(),10));
						$attachment['preserve'] = true;
						if($this->htmlExists($parent)){
							if($this->aParts[$parent]['mixed']){
								$this->aParts[$parent]['html_body'].='<img src="cid:'.$attachment['cid'].'" />';
							}else{
								$this->aParts[$parent]['html_body']='<img src="cid:'.$attachment['cid'].'" />';
							}
							
						}
					}
					$this->aParts[$parent]['attachments'][] = $attachment;
				break;
			
			/************************************************************************
			 *														MULTI PART																*
			 *																																			*
			 ************************************************************************
			 */	
			case 'multipart/signed':
					if(!isset($this->aParts[$parent])){
						$parent.='.1';
					}
					$this->partDistribution($parent,$partID);

			break;
			case 'multipart/appledouble':
						$partID .='.2';
						$part = mailparse_msg_get_part($this->mail, $partID);
						$info = mailparse_msg_get_part_data($part);
						
						$this->aParts[$parent]['attachments'][] = $this->parseAttachment($part, $info, $partID);
			break;
			//Move html/plain parts to higher structure
			case 'multipart/related': 
			case 'multipart/relative': 
			case 'multipart/alternative': 
			case 'multipart/mixed':
				$this->partDistribution($parent,$partID);
				break;
			//Move only attachments to higher structure(FIXME ? merge)
			case 'multipart/digest':
				$this->aParts[$parent]['attachments'] = $this->aParts[$partID]['attachments'];
				break;
			case 'multipart/report':
				if(!$this->aParts[$partID]['html_body'])
					$this->aParts[$partID]['html_body'] = Tools::text2html($this->aParts[$partID]['plain_body'], true).
																								($this->aParts[$partID]['delivery-status']?'<hr/>':'').
																								Tools::text2html($this->aParts[$partID]['delivery-status'], true);
				break;
			
		}
	}
	
	private function partDistribution($parent,$partID)
	{
		//Last level
		if($parent==$partID){
			return;
		}
		//Distribute part elements to higher part
		if($this->aParts[$parent]['mixed']){
			$this->aParts[$parent]['plain_body'] .= $this->aParts[$partID]['plain_body'];
			$this->aParts[$parent]['html_body'] .= $this->aParts[$partID]['html_body'];
		}else{
			if($this->aParts[$partID]['plain_body']){
				$this->aParts[$parent]['plain_body'] .= $this->aParts[$partID]['plain_body'];
			}
			if($this->aParts[$partID]['html_body']){
				$this->aParts[$parent]['html_body'] .= $this->aParts[$partID]['html_body'];
			}
		}
		if($this->aParts[$partID]['attachments']){
			if(!$this->aParts[$parent]['attachments']){
				$this->aParts[$parent]['attachments'] = array();
			}
			$this->aParts[$parent]['attachments'] = array_merge(
				$this->aParts[$partID]['attachments'],
				$this->aParts[$parent]['attachments']
			);
		}
		if($this->aParts[$partID]['signatures']){
			if(!$this->aParts[$parent]['signatures']){
				$this->aParts[$parent]['signatures'] = array();
			}
			$this->aParts[$parent]['signatures'] = array_merge(
				$this->aParts[$partID]['signatures'],
				$this->aParts[$parent]['signatures']
			);
		}
	}
	
	private function replaceCID()
	{

				/* Substitute cid URIs. */
		if ($this->ids) {

			$search	= array();
			$replace = array();
			if($this->aResult['attachments']) foreach ($this->aResult['attachments'] as $key => $attachment) {
			
				if ($attachment['cid'] === '')
					continue;

				$search[$i] = '/(<((?i)img)[^>]*((?i)src)[^=]*=[^"]*("?))(((?i)cid:)' . /*)*/
					preg_quote($attachment['cid'], '/') . ')(\4[^>]*>)/';
				$fullpath = $this->ids['account_id'].'/'.$this->ids['folder_id'].'/'.
					$this->ids['item_id'].'/'.$this->attIdPrefix.$attachment['part_id'];
				
				$url = 'http'.($_SERVER['HTTPS']=='ON'?'s':'').'://'.$_SERVER['HTTP_HOST'].str_replace(WEBMAIL_PHP,DOWNLOAD_PHP,$_SERVER['SCRIPT_NAME']);
				
				$replace[$i] = '\1' .
					$url . '?sid=' . encodeURL($this->ids['sid']) .
					'&amp;class=attachment' .
					'&amp;fullpath=' . encodeURL($fullpath) .
					'\7';

				$match = $search[$i];

				//Check if 
				if(preg_match($match,$this->aResult['html_body']) && !$attachment['preserve'])
					unset($this->aResult['attachments'][$key]);			
				//If not found in e-mail set as default attachment
				else
					unset($this->aResult['attachments'][$key]['cid']);
				
				$i++;
			}
			
			if ($search) {
					if (!is_string($this->aResult['html_body']))
							return;

					$this->aResult['html_body'] = preg_replace($search, $replace, $this->aResult['html_body']);
			
			}
		}

	}
	/**
		@brief Parse headers of the message.
		@param $info Information about the message part.
		@return The list of headers as an array of (header_name => header_value).
	 */
	private static function parseHeaders($info)
	{
		if (!isset($info['headers']))
			return array();

		$headers = $info['headers'];
		
		// fill result with default values (for the case they are not presented in $headers)
		$result = array('date' => -1,
							'from' => '',
							'to' => '',
							'cc' => '',
							'bcc' => '',
							'reply-to' => '',
							'subject' => '',
							'priority' => Item::NORMAL_PRIORITY,
							'custom-headers' => array(),
							'received'=>'');
							
		if($headers) foreach ($headers as $key => $value) {
			switch ($key) {
				case 'date':
					$date = @intval(strtotime($value));
					if ($date > 0)
						$result['date'] = date('Y-m-d H:i:s', $date);
					break;
				
				case 'received':
				case 'from':
				case 'to':
				case 'cc':
				case 'bcc':
				case 'reply-to':
				case 'subject':
				case 'message-id':
					$h = self::decodeMimeHeader($value);
					if(is_array($h)){
						$v = implode(',',$h);
					}else{
						$v = $h;
					}
					if($key=='from' || $key=='to'){
						$v = Tools::fixHeader($v);
					}else if ( $key=='subject' ){
						$v = trim($v);
					}
					$result[$key] = $v;
					break;

				case 'x-priority':
					$result['priority'] = self::decodePriority($value);
					break;
				
				// these headers shouldn't be sent to the output
				case 'received':
				case 'content-type':
					break;
				case 'mime-version':
					$result['mime'] = 1;
					break;
				
				default:
					$result['custom-headers'][$key] = self::decodeMimeHeader($value);
					break;
			}
		}
		return $result;
	}
	
	/**
		@param Get headers of a message.
	 */
	public function getHeaders($plain = false)
	{
		$part = mailparse_msg_get_part($this->mail, $this->startPart);
		$info = mailparse_msg_get_part_data($part);

		if ($plain)
			return $info['headers'];
		else
			return self::parseHeaders($info);
	}	
	
	/**
		@brief Parse information about an attachment.
		@param $part The attachment part.
		@param $info Information about the part.
		@param $partID ID of the part.
		@return Information about the attachment
		as an array of ('part_id', 'cid', 'type', 'name', 'size').
	 */
	private function parseAttachment($part, $info, $partID)
	{
		if(!$info){
			$info = mailparse_msg_get_part_data($part);
		}
		$size = $info['ending-pos-body']-$info['starting-pos-body'];

		if ($info['transfer-encoding']=='base64'){
			$size = $size *(3/4);
		}
		if($info['content-type']=='message/rfc822'){
			$part = mailparse_msg_get_part($this->mail,$partID.'.1');
			$messageRFC822Info = mailparse_msg_get_part_data($part);
		}
		
		$sName = self::getAttachmentName($info, $partID, $messageRFC822Info);
		$sContentType = $info['content-type']; 
		Tools::overrideContentType($sName,$sContentType);
	 	return array(
			'part_id' => $partID,
			'cid'	 => isset($info['content-id']) ? $info['content-id'] : '',
			'type'	=> $sContentType,
			'name'	=> $sName,
			'size'	=> $size,
		);
	}
	
	/**
		@brief Get the name of an attachment.
		@param $info Information about the attachment part.
	 */
	static public function getAttachmentName($info, $partID, $rfc822Info = false)
	{

	//Message RFC822 as attachment FIX( multipart/digest)
		if (isset($info['headers']['subject']))
			$name = self::decodeMimeHeader($info['headers']['subject']);
		if (!$name && isset($info['content-name']))
			$name = self::decodeMimeHeader($info['content-name']);
		if (!$name && isset($info['content-description']))
			$name = self::decodeMimeHeader($info['content-description']);
		if (!$name && isset($info['disposition-filename']))
			$name = self::decodeMimeHeader($info['disposition-filename']);
		if (!$name && isset($rfc822Info['headers']['subject'])){
			$name = self::decodeMimeHeader($rfc822Info['headers']['subject']);
		}
		if ($name)
			return $name;
		else
			return $partID.'.eml';
	}
	/**
		@brief Parse a body.
		@param $part The body part.
		@param $info Information about the part.
		@return The body.
	 */
	private function parseBody($part, $info, $partID)
	{
		
		if($this->file)
			@$data = mailparse_msg_extract_part_file($part, $this->file,NULL);
		else
			@$data = mailparse_msg_extract_part($part, $this->message,NULL);
		//UUencoded parts
		if(!isset($this->aParts[$partID]['headers']['mime']) && !isset($info['headers']['mime-version'])){
				$this->aUUEncodedContent = array();
				$subPart = 1;
				$offset = 0;
				$stop = 0;
				while(preg_match("#(begin) ([0-9]{1,}) ([^\r]{0,})#si",$data,$matches,PREG_OFFSET_CAPTURE,$offset)){
					
					//Fix : mailparse sometimes cut the part before last end
					if(!preg_match("#(end)#si",$data,$pom)){
						$data.='\rend';
					}
					$this->aUUAttachments[$this->startPart.'.uuencode.'.$subPart][] = $this->parseUUEncodedAttachment($data,$matches,$offset,$subPart);	 
				}
		}
		// If extended ASCII
	//	if (Tools::isabovechar($data)){
			 $result = Tools::my_iconv($info['content-charset'], 'UTF-8//IGNORE', $data);
//		}
		//Parse charset manually if no result
		if(!$result){
			preg_match("#charset(\s|\t)?+=(\s|\t)?+(\"|\'})?([^\"^\'^\s^\r^\n^;]{1,})(;)?+(\"|\'})?#i",$info['headers']['content-type'],$charsetMatch);
			$charsetFromContentType = $charsetMatch[4];
			if($charsetFromContentType
			&& $info['content-charset']
			&& $info['content-charset']!=$charsetFromContentType){
				$result = Tools::my_iconv(strtolower($charsetFromContentType), 'UTF-8//IGNORE', $data);
			}
		}
		return $result;
	}

	/**
		@brief Decode mime header.
		@param $header Header to be decoded.
		@return Decoded header.
	 */
	public static function decodeMimeHeader($header,$debug = false)
	{
		$converted = array();
		if(!is_array($header)){
			$headers[0] = $header;
		}else{
			$headers = $header;
		}
		foreach($headers as $key => $sheader){
			$converted[$key] = '';
			if($headerArr = imap_mime_header_decode($sheader)){
				foreach ($headerArr as $item) {
					if ($item->charset == 'default' /*|| strtolower($item->charset) == 'us-ascii'*/){
						 if (!Tools::isabovechar($item->text)){
							$converted[$key] .= $item->text;
						 }else{
					 		$converted[$key] .= Tools::my_iconv($_SESSION['DEFAULTCHARSET'], 'UTF-8//IGNORE', $item->text);
						}
					}
					else{
						$converted[$key] .= Tools::my_iconv($item->charset, 'UTF-8//IGNORE', $item->text);
					}
				}
			}
		}
		if(is_array($converted) && count($converted)==1){
			$converted = reset($converted);
		}
		if(!$converted){
			$converted = '';
		}
		return $converted;
	}
	/**
		@brief Decode message priority.
		@param $priority Message priority.
		@return Decoded priority
	 */
	public static function decodePriority($priority)
	{
		ereg('^([0-9]{1,})',$priority,$regs);
		@$tmp = intval(reset($regs));
		return $tmp ? $tmp : Item::NORMAL_PRIORITY;
	}
	
	/**
		@brief Parse addresses from a header (to, from, cc).
		@param $header Header to be parsed.
		@return Array containing addresses of a header.
	 */
	public static function parseAddresses($header)
	{
		return @mailparse_rfc822_parse_addresses($header);
	}

	/**
		@brief Detects whether a message structure contains attachments.
		@param $structure Structure of the message
		@return TRUE or FALSE
	 */
	public static function hasAttachments($structure)
	{
		// find out whether any attachment without CID exists
		if($structure['attachments']) foreach ($structure['attachments'] as $attachment) {
			if ($attachment['cid'] === '' || !isset($attachment['cid']))
				return true;
		}
		return false;
	}
	/**
		@brief Detects whether a message structure contains HTML body.
		@param $structure Structure of the message
		@return TRUE or FALSE
	 */
	public static function hasHTMLBody($structure)
	{
		// find out whether HTML body exists
		if (isset($structure['html_body']) && $structure['html_body']!='')
			return true;
		else
			return false;
	}
	/**
		@brief Detects whether a message is SMime signed - if so, verifies the signature.
		@param $structure Structure of the message
		@param $message Message itself.
		@return 1 - message is not subscribed, 2 - message is subscribed and valid,
			3 - messaeg is subscribed and subscription is not valid
	 */
	public static function sMimeStatus($structure, &$message, $file = false)
	{
		if($structure['encrypted'] == true){
		 return $structure['error']?3:2;
		}
		if (!isset($structure['signatures']))
			return 1;
		foreach ($structure['signatures'] as $item) {
			if ($item['type'] == 'application/x-pkcs7-signature'
				 || $item['type'] == 'application/pkcs7-signature'
				 || $item['type'] == 'application/pkcs7-mime'){
				if (SMime::verify($file))
					return 2;
				else
					return 3;
			}
		}
		
		// no signature found, return 0
		return 1;
	}
	
	/**
		@brief Send a message part to the browser.
		@param $partID ID of the part.
	 */
	public function sendPart($partID,$sForceName = false)
	{
		$i = 0;
		if (!isset($this->partPointer[$partID])) {
			if ($result = $this->getUUEncodedPart($partID)){
					/* Send attachment data to output. */
		 		
				 Tools::sendFileHeaders($sForceName?$sForceName:$result['filename'],$result['size'],$result['type']);
				echo $result['content'];
				return;	
					
			} else if($result = $this->getTnefPart($partID)){
			
				Tools::sendFileHeaders($sForceName?$sForceName:$result['filename'],$result['size'],$result['type']);
						
				echo $result['content'];
				return;
			}
			return $this->recursivelyProcessEncryptedAttachments('sendPart', $partID);
		}

		$part = mailparse_msg_get_part($this->mail, $partID);
		$info = mailparse_msg_get_part_data($part);
		$fileName = $sForceName?$sForceName:self::getAttachmentName($info, $partID);
		
		$length = 0;
		
		/* Send attachment data to output. */
		Tools::sendFileHeaders($fileName, $length, $info['content-type']);

		if($this->file)
			@mailparse_msg_extract_part_file($part,$this->file);
		else
			@mailparse_msg_extract_part($part,$this->message);

	}
	
	public function getPart($partID,&$params = array())
	{

		if (!isset($this->partPointer[$partID])) {
			if ($result = $this->getUUEncodedPart($partID)){
					$params['info']['content-name'] = $result['filename'];
					$params['info']['content-type'] = $result['type'];
				 $params['info']['transfer-encoding'] = 'uu';
				 // Return attachment data 
				return $result['content'];
			}else if($result = $this->getTnefPart($partID)){
				$params['info']['content-name'] = $result['filename'];
				$params['info']['content-type'] = $result['type'];
				$params['info']['transfer-encoding'] = 'tnef';
					
				
				return $result['content'];
			}

			return $this->recursivelyProcessEncryptedAttachments('getPart', $partID, $params);
		}


		$part = mailparse_msg_get_part($this->mail, $partID);
		$info = mailparse_msg_get_part_data($part);

		$params['info'] = $info;

		if($this->file)
			$data = @mailparse_msg_extract_part_file($part,$this->file,NULL);
		else
			$data = @mailparse_msg_extract_part($part,$this->message,NULL);

		return $data;		
	}	
	
	public function getPartFile($partID,&$params = array())
	{

		$fName = Tools::randomFilename();
		
		if (!isset($this->partPointer[$partID])) {
			if ($result = $this->getUUEncodedPart($partID)){
					$params['info']['content-name'] = $result['filename'];
					$params['info']['content-type'] = $result['type'];
				 $params['info']['transfer-encoding'] = 'uu';
				 file_put_contents($fName,$result['content']);
				 // Return attachment data 
				return $fName;
			}else if($result = $this->getTnefPart($partID)){
				$params['info']['content-name'] = $result['filename'];
				$params['info']['content-type'] = $result['type'];
				$params['info']['transfer-encoding'] = 'tnef';
				
				file_put_contents($fName,$result['content']);
					
				return $fName;
			}

			return $this->recursivelyProcessEncryptedAttachments('getPart', $partID, $params);
		}

		$part = mailparse_msg_get_part($this->mail, $partID);
		$info = mailparse_msg_get_part_data($part);

		$params['info'] = $info;

		if(!is_dir(dirname($fName)))
			System::mkdir_r(dirname($fName));
		$this->attachmentFile = fopen($fName,'wb+');
		
		if($this->file)
			@mailparse_msg_extract_part_file($part,$this->file,array($this,"put_file"));
		else
			@mailparse_msg_extract_part($part,$this->message,array($this,"put_file"));

		@fclose($fName);

		return $fName;		
	}		

	private function getUUEncodedPart($partID){


		//Parse message body to get attachments
		$this->parse();

		if(!$aAttachment = $this->aUUAttachments[$partID][0])
			return false;
					
		$result['filename'] = $aAttachment['name'];
		$result['content'] = $this->aUUEncodedContent[$partID];
		$result['size'] = strlen($result['content']);
		$result['type'] = $aAttachment['type'];



		return $result;
	}	
	private function parseTnefPart($part,$parent,$partID)
	{
		$name = Tools::randomFileName();
		if($this->file){
			$data = @mailparse_msg_extract_part_file($part, $this->file,NULL);
		}else{
			$data = @mailparse_msg_extract_part($part, $this->message,NULL);
		}
		file_put_contents($name,$data);

				$oTnef = tnef_parse_file($name);
				$iTnef = 0;
				$iRTF = 0;
				$attachments = tnef_list_attachments($oTnef);
				$body = tnef_get_body($oTnef);
				if($body){
					if($body['html'])
						$this->aParts[$partID]['html_body'].=$body['html'];
					else	if($body['text']) 
							$this->aParts[$partID]['plain_body'].=$body['text'];

					if($body['rtf'])
						$this->aParts[$parent]['attachments'][] = array(
																												 'part_id' => $partID.'.tnef.rtf.'.(++$iRTF),
																												 'type'	=> 'text/rtf',
																												 'name'	=> 'attachment - '.$iTnef.'.rtf',
																												 'size'	=> strlen($body['rtf'][0]));
				}
				
				if ($attachments) foreach($attachments as $attachment){
					 	 $data = tnef_get_attachment($oTnef,$attachment['name']);
						 $this->aParts[$parent]['attachments'][] = array(
																											 'part_id' => $partID.'.tnef.'.(++$iTnef),
																											 'type'	=> 'application/octet-stream',
																											 'name'	=> $attachment['name'],
																											 'size'	=> $attachment['size']);
				}					
				unset($oTnef);
				unlink($name);	
	}
	
	private function getTnefPart($partID){
		
		$rtf = false;

		if(strpos($partID,".tnef.rtf.")!==false){
			$partID = explode(".tnef.rtf.",$partID);
			$rtf = true;
		}

		if(strpos($partID,".tnef.")!==false){
			$partID = explode(".tnef.",$partID);
			
		}

		$rtfID = $partID[1];
		$partID = $partID[0];

		$name = Tools::randomFileName();
		$part = mailparse_msg_get_part($this->mail, $partID);	 

		$result = false;	
				
		if($this->file)
			$data = @mailparse_msg_extract_part_file($part, $this->file,NULL);
		else
			$data = @mailparse_msg_extract_part($part, $this->message,NULL);

		
		file_put_contents($name,$data);
				
		$oTnef = tnef_parse_file($name);
		$iTnef = 0;

		if($rtf){
		
			$body = tnef_get_body($oTnef);

			$result = array('part_id' => $partID.'.tnef.rtf.'.$rtfIF,
											 'type'	=> 'text/rtf',
											 'filename'	=> 'attachment - '.$partID.'.rtf',
											 'size'	=> strlen($body['rtf'][0]),
											 'content' => $body['rtf'][0]);
			
			unset($oTnef);
			unlink($name);
			return $result;
		}
		

		$attachments = tnef_list_attachments($oTnef);

		if ($attachments) foreach($attachments as $attachment)
				if((++$iTnef)==$rtfID){
					$data = tnef_get_attachment($oTnef,$attachment['name']);
			 		$result = array('part_id' => $partID.'.tnef.'.($iTnef),
											 'type'	=> 'application/octet-stream',
											 'filename'	=> $attachment['name'],
											 'size'	=> '',
											 'content' => $data);
		}

		unset($oTnef);
		unlink($name);
		
		return $result;
	}	
		
	public function parseUUEncodedAttachment(&$body,$matches,&$offset,&$partID)
	{

			preg_match("#(end)#s",$body,$endmatches,PREG_OFFSET_CAPTURE,$offset);
			
			
			$start = $matches[3][1]+strlen($matches[3][0]);
			$length = $endmatches[1][1] - $start;	
			$offset = $matches[1][1];

			$content = trim(str_replace("\r","",substr($body,$start,$length)));

			$this->aUUEncodedContent[$this->startPart.'.uuencode.'.$partID] = @convert_uudecode($content);
			$size = strlen($this->aUUEncodedContent[$this->startPart.'.uuencode.'.$partID]);

			$body = substr_replace($body,"",$matches[1][1],$endmatches[1][1]+ strlen($endmatches[1][0]) - $matches[1][1]);
			$aAttachment =	array(
												'part_id' => $this->startPart.'.uuencode.'.$partID++,
												'cid'	 => '',
												'type'	=> 'application/octet-stream',
												'name'	=> $matches[3][0],
												'size'	=> $size,
											);
			return $aAttachment;
	}
	/**
		@brief Recursively decrypt encrypted attachment (specified by partID) and
		call method to process the decrypted part.
		@param $funcName Name of the method to be called
		@param $partID ID of the part.
		@param $params Optional parameters for the method to be called
	 */
	private function recursivelyProcessEncryptedAttachments($methodName, $partID, &$params = null)
	{
		// find out if recursively SMimem encoded attachment is required
		$needle = '.decrypt.';

		$delimiter = strpos($partID, $needle);
		if ($delimiter === false)
			throw new Exc('mailparse_get_part',$partID);
		
		$parser = $this->decryptPart(substr($partID, 0, $delimiter));
		if ($parser === false)
			throw new Exc('mailparse_get_part',$partID);
			
		return $parser->$methodName(substr($partID,$delimiter + strlen($needle)), $params);
	}	
	/**
		@brief Process all attachments of a message. If an attachment is SMime encoded
		part, decode it and process the parsing function for the decoded part. If there
		is only one encoded part in whole message, store the decoded structure into
		$this->sMimeMessage variable.
		@param[in,out] $structure Message strucrure of the MailParse function.
	 */
	private function checkSMimeParts(&$structure)
	{

		if (isset($structure['attachments'])) {

			foreach ($structure['attachments'] as $key => $att) {

				// encrypted part found, process it recursively
				if ($att['type'] == 'application/x-pkcs7-mime' 
				|| $att['type'] == 'application/pkcs7-mime') {

					$parser = $this->decryptPart($att['part_id'], $store);

					if ($parser !== false) {
						$part = $parser->parse();

						// recursively add all encrypted attachments with special ID notation
						if (isset($part['attachments'])) {
							foreach ($part['attachments'] as &$partAtt) {
								$aID = strval($att['part_id']) . '.decrypt.' . strval($partAtt['part_id']);
								$partAtt['part_id'] = $aID;
								$structure['attachments'][] = $partAtt;
							}
						}
						
						// handle special case - if there is no body of the message and the first
						// attachment is being processed (the message is whole encrypted), store
						// the parsed structure of the decoded message into $this->sMimeMessage 
						// variable which will then be made available for calling function
						if ($key == 0 && $this->attIdPrefix == '' && !$structure['plain_body'] && !$structure['html_body']) {
								$this->sMimeMessage = $part;
						}else{
							$this->sMimeMessage['error'] = 1;
						}
					}else{
						$this->sMimeMessage['error'] = 1;
					}
				}
			}
		}
	}
	
	/**
		@brief Get MailParse object of SMime encoded message part.
		@param $partID ID of the SMime encoded message part.
		@return MailParse object if the part can be decoded, false otherwise.
	 */
	private function decryptPart($partID) {
		//echo $partID;
		//$part = mailparse_msg_get_part( $this->mail, $partID );
		if($this->file){
			$encrypted = file_get_contents($this->file);
		}else{
			$encrypted = $this->message;
		}
		
		if ( SMime::decodeMessage( $encrypted ) ) {
			
			$attIdPrefix = $this->attIdPrefix . strval( $partID ) . '.decrypt.';
			$parser = new MailParse(false, $this->ids, $this->decodeMime, $attIdPrefix, $partID, $encrypted );
			return $parser;
		}
		return false;
	}

	function put_file($string)
	{
		fwrite($this->attachmentFile,$string);
	}

}
?>
