<?php

/**
  @class Variable
  @brief Abstract class for variables management.
 */
abstract class Variable {
	/** @brief Reference to the array. */
	protected $array;

	/**
	  @brief A private constructor.
	  Prevent direct creation of object.
	 */
	protected function __construct(&$array)
	{
		$this->array = &$array;
	}

	/**
	  @brief Check if value is email.
	  @param $value The value.
	  @return True on success, false on failure.
	 */
	public static function isEmail($value)
	{
		return preg_match('/^([_a-zA-Z0-9-]+)(\.[_a-zA-Z0-9-]+)*@([a-zA-Z0-9-]+)(\.[a-zA-Z0-9-]+)*(\.[a-z]{2,4})$/', $value);
	}

	/**
	  @brief Get a variable as email.
	  @param $name Name of the variable.
	  @return Value of the variable as email on success, null on failure.
	 */
	public function getEmail($name)
	{
		$value = $this->$name;
		return self::isEmail($value) ? $value : null;
	}

	/**
	  @brief Check if value of a variable is date.
	  @param $value The value.
	  @return True on success, false on failure.
	 */
	public static function isDate($value)
	{
		return preg_match('/^[0-9]{4}[-.\/][0-9]{1,2}[-.\/][0-9]{1,2}$/', $value);
	}

	/**
	  @brief Get a variable as date string.
	  @param $name Name of the variable.
	  @return Value of the variable as date string on success, null on failure.
	 */
	public function getDate($name)
	{
		$value = $this->$name;
		return self::isDate($value) ? $value : null;
	}

	/**
	  @brief Check if value of a variable is integer.
	  @param $value The value.
	  @return True on success, false on failure.
	 */
	public static function isInt($value)
	{
		return preg_match('/^-?[0-9]+$/', $value);
	}

	/**
	  @brief Get a variable as integer.
	  @param $name Name of the variable.
	  @return Value of the variable as integer on success, null on failure.
	 */
	public function getInt($name)
	{
		$value = $this->$name;
		return self::isInt($value) ? intval($value) : null;
	}

	/**
	  @brief Check if value of a variable is float.
	  @param $value The value.
	  @return True on success, false on failure.
	 */
	public static function isFloat($value)
	{
		return preg_match('/^-?[0-9]*([.,][0-9]+)?$/', $value);
	}

	/**
	  @brief Get a variable as float.
	  @param $name Name of the variable.
	  @return Value of the variable as float on success, null on failure.
	 */
	public function getFloat($name)
	{
		$value = $this->$name;
		return self::isFloat($value) ? floatval($value) : null;
	}

	/**
	  @brief Get a varible.
	  @param $name Name of the variable.
	  @return Value of the variable on success, null on failure.
	 */
	public function __get($name)
	{
		return isset($this->array[$name]) ? $this->array[$name] : null;
	}

	/**
	  @brief Set a varible.
	  @param $name Name of the variable.
	  @param $value Value of the variable.
	 */
	public function __set($name, $value)
	{
		$this->array[$name] = $value;
	}
}

?>
